"""
Python module generated from Java source file org.joml.Vector4f

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import Externalizable
from java.io import IOException
from java.io import ObjectInput
from java.io import ObjectOutput
from java.text import DecimalFormat
from java.text import NumberFormat
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Vector4f(Externalizable, Cloneable, Vector4fc):
    """
    Contains the definition of a Vector comprising 4 floats and associated
    transformations.

    Author(s)
    - F. Neurath
    """

    def __init__(self):
        """
        Create a new Vector4f of `(0, 0, 0, 1)`.
        """
        ...


    def __init__(self, v: "Vector4fc"):
        """
        Create a new Vector4f with the same values as `v`.

        Arguments
        - v: the Vector4fc to copy the values from
        """
        ...


    def __init__(self, v: "Vector4dc"):
        """
        Create a new Vector4f with the same values as `v`.
        
        Note that due to the given vector `v` storing the components in double-precision,
        there is the possibility to lose precision.

        Arguments
        - v: the Vector4dc to copy the values from
        """
        ...


    def __init__(self, v: "Vector4ic"):
        """
        Create a new Vector4f with the same values as `v`.

        Arguments
        - v: the Vector4ic to copy the values from
        """
        ...


    def __init__(self, v: "Vector3fc", w: float):
        """
        Create a new Vector4f with the first three components from the
        given `v` and the given `w`.

        Arguments
        - v: the Vector3fc
        - w: the w component
        """
        ...


    def __init__(self, v: "Vector3ic", w: float):
        """
        Create a new Vector4f with the first three components from the
        given `v` and the given `w`.

        Arguments
        - v: the Vector3ic
        - w: the w component
        """
        ...


    def __init__(self, v: "Vector2fc", z: float, w: float):
        """
        Create a new Vector4f with the first two components from the
        given `v` and the given `z`, and `w`.

        Arguments
        - v: the Vector2fc
        - z: the z component
        - w: the w component
        """
        ...


    def __init__(self, v: "Vector2ic", z: float, w: float):
        """
        Create a new Vector4f with the first two components from the
        given `v` and the given `z`, and `w`.

        Arguments
        - v: the Vector2ic
        - z: the z component
        - w: the w component
        """
        ...


    def __init__(self, d: float):
        """
        Create a new Vector4f and initialize all four components with the given value.

        Arguments
        - d: the value of all four components
        """
        ...


    def __init__(self, x: float, y: float, z: float, w: float):
        """
        Create a new Vector4f with the given component values.

        Arguments
        - x: the x component
        - y: the y component
        - z: the z component
        - w: the w component
        """
        ...


    def __init__(self, xyzw: list[float]):
        """
        Create a new Vector4f and initialize its four components from the first
        four elements of the given array.

        Arguments
        - xyzw: the array containing at least four elements
        """
        ...


    def __init__(self, buffer: "ByteBuffer"):
        """
        Create a new Vector4f and read this vector from the supplied ByteBuffer
        at the current buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the vector is read, use .Vector4f(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y, z, w` order

        See
        - .Vector4f(int, ByteBuffer)
        """
        ...


    def __init__(self, index: int, buffer: "ByteBuffer"):
        """
        Create a new Vector4f and read this vector from the supplied ByteBuffer
        starting at the specified absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: values will be read in `x, y, z, w` order
        """
        ...


    def __init__(self, buffer: "FloatBuffer"):
        """
        Create a new Vector4f and read this vector from the supplied FloatBuffer
        at the current buffer FloatBuffer.position() position.
        
        This method will not increment the position of the given FloatBuffer.
        
        In order to specify the offset into the FloatBuffer at which
        the vector is read, use .Vector4f(int, FloatBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y, z, w` order

        See
        - .Vector4f(int, FloatBuffer)
        """
        ...


    def __init__(self, index: int, buffer: "FloatBuffer"):
        """
        Create a new Vector4f and read this vector from the supplied FloatBuffer
        starting at the specified absolute buffer position/index.
        
        This method will not increment the position of the given FloatBuffer.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: values will be read in `x, y, z, w` order
        """
        ...


    def x(self) -> float:
        ...


    def y(self) -> float:
        ...


    def z(self) -> float:
        ...


    def w(self) -> float:
        ...


    def xyz(self, dest: "Vector3f") -> "Vector3f":
        """
        Copy the `(x, y, z)` components of `this` into the supplied `dest` vector
        and return it.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def xyz(self, dest: "Vector3d") -> "Vector3d":
        """
        Copy the `(x, y, z)` components of `this` into the supplied `dest` vector
        and return it.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def xy(self, dest: "Vector2f") -> "Vector2f":
        """
        Copy the `(x, y)` components of `this` into the supplied `dest` vector
        and return it.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def xy(self, dest: "Vector2d") -> "Vector2d":
        """
        Copy the `(x, y)` components of `this` into the supplied `dest` vector
        and return it.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def set(self, v: "Vector4fc") -> "Vector4f":
        """
        Set this vector to the values of the given `v`.

        Arguments
        - v: the vector whose values will be copied into this

        Returns
        - this
        """
        ...


    def set(self, v: "Vector4ic") -> "Vector4f":
        """
        Set this vector to the values of the given `v`.

        Arguments
        - v: the vector whose values will be copied into this

        Returns
        - this
        """
        ...


    def set(self, v: "Vector4dc") -> "Vector4f":
        """
        Set this vector to the values of the given `v`.
        
        Note that due to the given vector `v` storing the components in double-precision,
        there is the possibility to lose precision.

        Arguments
        - v: the vector whose values will be copied into this

        Returns
        - this
        """
        ...


    def set(self, v: "Vector3fc", w: float) -> "Vector4f":
        """
        Set the first three components of this to the components of
        `v` and the last component to `w`.

        Arguments
        - v: the Vector3fc to copy
        - w: the w component

        Returns
        - this
        """
        ...


    def set(self, v: "Vector3dc") -> "Vector4f":
        """
        Set this vector to the `(x, y, z)` components of the given `v`.
        
        Note that due to the given vector `v` storing the components in double-precision,
        there is the possibility to lose precision.

        Arguments
        - v: the vector whose values will be copied into this

        Returns
        - this
        """
        ...


    def set(self, v: "Vector3ic", w: float) -> "Vector4f":
        """
        Set the first three components of this to the components of
        `v` and the last component to `w`.

        Arguments
        - v: the Vector3ic to copy
        - w: the w component

        Returns
        - this
        """
        ...


    def set(self, v: "Vector2fc", z: float, w: float) -> "Vector4f":
        """
        Sets the first two components of this to the components of given `v`
        and last two components to the given `z`, and `w`.

        Arguments
        - v: the Vector2fc
        - z: the z component
        - w: the w component

        Returns
        - this
        """
        ...


    def set(self, v: "Vector2ic", z: float, w: float) -> "Vector4f":
        """
        Sets the first two components of this to the components of given `v`
        and last two components to the given `z`, and `w`.

        Arguments
        - v: the Vector2ic
        - z: the z component
        - w: the w component

        Returns
        - this
        """
        ...


    def set(self, d: float) -> "Vector4f":
        """
        Set the x, y, z, and w components to the supplied value.

        Arguments
        - d: the value of all four components

        Returns
        - this
        """
        ...


    def set(self, x: float, y: float, z: float, w: float) -> "Vector4f":
        """
        Set the x, y, z, and w components to the supplied values.

        Arguments
        - x: the x component
        - y: the y component
        - z: the z component
        - w: the w component

        Returns
        - this
        """
        ...


    def set(self, x: float, y: float, z: float) -> "Vector4f":
        """
        Set the x, y, z components to the supplied values.

        Arguments
        - x: the x component
        - y: the y component
        - z: the z component

        Returns
        - this
        """
        ...


    def set(self, d: float) -> "Vector4f":
        """
        Set the x, y, z, and w components to the supplied value.

        Arguments
        - d: the value of all four components

        Returns
        - this
        """
        ...


    def set(self, x: float, y: float, z: float, w: float) -> "Vector4f":
        """
        Set the x, y, z, and w components to the supplied values.

        Arguments
        - x: the x component
        - y: the y component
        - z: the z component
        - w: the w component

        Returns
        - this
        """
        ...


    def set(self, xyzw: list[float]) -> "Vector4f":
        """
        Set the four components of this vector to the first four elements of the given array.

        Arguments
        - xyzw: the array containing at least four elements

        Returns
        - this
        """
        ...


    def set(self, buffer: "ByteBuffer") -> "Vector4f":
        """
        Read this vector from the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the vector is read, use .set(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y, z, w` order

        Returns
        - this

        See
        - .set(int, ByteBuffer)
        """
        ...


    def set(self, index: int, buffer: "ByteBuffer") -> "Vector4f":
        """
        Read this vector from the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: values will be read in `x, y, z, w` order

        Returns
        - this
        """
        ...


    def set(self, buffer: "FloatBuffer") -> "Vector4f":
        """
        Read this vector from the supplied FloatBuffer at the current
        buffer FloatBuffer.position() position.
        
        This method will not increment the position of the given FloatBuffer.
        
        In order to specify the offset into the FloatBuffer at which
        the vector is read, use .set(int, FloatBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y, z, w` order

        Returns
        - this

        See
        - .set(int, FloatBuffer)
        """
        ...


    def set(self, index: int, buffer: "FloatBuffer") -> "Vector4f":
        """
        Read this vector from the supplied FloatBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given FloatBuffer.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: values will be read in `x, y, z, w` order

        Returns
        - this
        """
        ...


    def setFromAddress(self, address: int) -> "Vector4f":
        """
        Set the values of this vector by reading 4 float values from off-heap memory,
        starting at the given address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap memory address to read the vector values from

        Returns
        - this
        """
        ...


    def setComponent(self, component: int, value: float) -> "Vector4f":
        """
        Set the value of the specified component of this vector.

        Arguments
        - component: the component whose value to set, within `[0..3]`
        - value: the value to set

        Returns
        - this

        Raises
        - IllegalArgumentException: if `component` is not within `[0..3]`
        """
        ...


    def get(self, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def get(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def get(self, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def get(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getToAddress(self, address: int) -> "Vector4fc":
        ...


    def sub(self, v: "Vector4fc") -> "Vector4f":
        """
        Subtract the supplied vector from this one.

        Arguments
        - v: the vector to subtract

        Returns
        - this
        """
        ...


    def sub(self, x: float, y: float, z: float, w: float) -> "Vector4f":
        """
        Subtract `(x, y, z, w)` from this.

        Arguments
        - x: the x component to subtract
        - y: the y component to subtract
        - z: the z component to subtract
        - w: the w component to subtract

        Returns
        - this
        """
        ...


    def sub(self, v: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def sub(self, x: float, y: float, z: float, w: float, dest: "Vector4f") -> "Vector4f":
        ...


    def add(self, v: "Vector4fc") -> "Vector4f":
        """
        Add the supplied vector to this one.

        Arguments
        - v: the vector to add

        Returns
        - this
        """
        ...


    def add(self, v: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def add(self, x: float, y: float, z: float, w: float) -> "Vector4f":
        """
        Increment the components of this vector by the given values.

        Arguments
        - x: the x component to add
        - y: the y component to add
        - z: the z component to add
        - w: the w component to add

        Returns
        - this
        """
        ...


    def add(self, x: float, y: float, z: float, w: float, dest: "Vector4f") -> "Vector4f":
        ...


    def fma(self, a: "Vector4fc", b: "Vector4fc") -> "Vector4f":
        """
        Add the component-wise multiplication of `a * b` to this vector.

        Arguments
        - a: the first multiplicand
        - b: the second multiplicand

        Returns
        - this
        """
        ...


    def fma(self, a: float, b: "Vector4fc") -> "Vector4f":
        """
        Add the component-wise multiplication of `a * b` to this vector.

        Arguments
        - a: the first multiplicand
        - b: the second multiplicand

        Returns
        - this
        """
        ...


    def fma(self, a: "Vector4fc", b: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def fma(self, a: float, b: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulAdd(self, a: "Vector4fc", b: "Vector4fc") -> "Vector4f":
        """
        Add the component-wise multiplication of `this * a` to `b`
        and store the result in `this`.

        Arguments
        - a: the multiplicand
        - b: the addend

        Returns
        - this
        """
        ...


    def mulAdd(self, a: float, b: "Vector4fc") -> "Vector4f":
        """
        Add the component-wise multiplication of `this * a` to `b`
        and store the result in `this`.

        Arguments
        - a: the multiplicand
        - b: the addend

        Returns
        - this
        """
        ...


    def mulAdd(self, a: "Vector4fc", b: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulAdd(self, a: float, b: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mul(self, v: "Vector4fc") -> "Vector4f":
        """
        Multiply this vector component-wise by another Vector4f.

        Arguments
        - v: the other vector

        Returns
        - this
        """
        ...


    def mul(self, v: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def div(self, v: "Vector4fc") -> "Vector4f":
        """
        Divide this vector component-wise by another Vector4f.

        Arguments
        - v: the vector to divide by

        Returns
        - this
        """
        ...


    def div(self, v: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mul(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the given matrix mat with this vector and store the result in
        `this`.

        Arguments
        - mat: the matrix to multiply the vector with

        Returns
        - this
        """
        ...


    def mul(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulAffine(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the given affine matrix `mat` with this vector.
        
        This method only works if the given matrix _only_ represents an affine transformation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the affine matrix to multiply the vector with

        Returns
        - this
        """
        ...


    def mulAffine(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulTranspose(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the transpose of the given matrix `mat` with this vector and store the result in
        `this`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with

        Returns
        - this
        """
        ...


    def mulTranspose(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulAffineTranspose(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the transpose of the given affine matrix `mat` with this vector.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the affine matrix whose transpose to multiply the vector with

        Returns
        - this
        """
        ...


    def mulAffineTranspose(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulGenericTranspose(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the transpose of the given matrix `mat` with this vector.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the affine matrix whose transpose to multiply the vector with

        Returns
        - this
        """
        ...


    def mulGenericTranspose(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulTranslation(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the given matrix `mat`, which is assumed to only contain translation, with this vector.
        
        This method only works if the given matrix _only_ represents a translation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the affine matrix to multiply the vector with

        Returns
        - this
        """
        ...


    def mulTranslation(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulGeneric(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the given matrix `mat` with this vector.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with

        Returns
        - this
        """
        ...


    def mulGeneric(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mul(self, mat: "Matrix4x3fc") -> "Vector4f":
        """
        Multiply the given matrix mat with this vector.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply the vector with

        Returns
        - this
        """
        ...


    def mul(self, mat: "Matrix4x3fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulGeneric(self, mat: "Matrix4x3fc") -> "Vector4f":
        """
        Multiply the given matrix `mat` with this vector.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with

        Returns
        - this
        """
        ...


    def mulGeneric(self, mat: "Matrix4x3fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulTranslation(self, mat: "Matrix4x3fc") -> "Vector4f":
        """
        Multiply the given matrix `mat`, which is assumed to only contain translation, with this vector.
        
        This method only works if the given matrix _only_ represents a translation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with

        Returns
        - this
        """
        ...


    def mulTranslation(self, mat: "Matrix4x3fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulProject(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the given matrix `mat` with this vector and perform perspective division.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProject(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulProject(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulProjectGeneric(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the given matrix `mat` with this vector and perform perspective division.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProjectGeneric(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulProjectGeneric(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulProjectTranslation(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the given matrix `mat`, which is assumed to only contain translation,
        with this vector and perform perspective division.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProjectTranslation(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulProjectTranslation(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulProjectAffine(self, mat: "Matrix4fc") -> "Vector4f":
        """
        Multiply the given affine matrix `mat`, with this vector and perform perspective division.
        
        This method only works if the given matrix _only_ represents an affine transformation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProjectAffine(self, mat: "Matrix4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def mulProjectAffine(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mul(self, scalar: float) -> "Vector4f":
        """
        Multiply all components of this vector by the given scalar
        value.

        Arguments
        - scalar: the scalar to multiply by

        Returns
        - this
        """
        ...


    def mul(self, scalar: float, dest: "Vector4f") -> "Vector4f":
        ...


    def mul(self, x: float, y: float, z: float, w: float) -> "Vector4f":
        """
        Multiply the components of this vector by the given scalar values and store the result in `this`.

        Arguments
        - x: the x component to multiply by
        - y: the y component to multiply by
        - z: the z component to multiply by
        - w: the w component to multiply by

        Returns
        - this
        """
        ...


    def mul(self, x: float, y: float, z: float, w: float, dest: "Vector4f") -> "Vector4f":
        ...


    def div(self, scalar: float) -> "Vector4f":
        """
        Divide all components of this vector by the given scalar
        value.

        Arguments
        - scalar: the scalar to divide by

        Returns
        - this
        """
        ...


    def div(self, scalar: float, dest: "Vector4f") -> "Vector4f":
        ...


    def div(self, x: float, y: float, z: float, w: float) -> "Vector4f":
        """
        Divide the components of this vector by the given scalar values and store the result in `this`.

        Arguments
        - x: the x component to divide by
        - y: the y component to divide by
        - z: the z component to divide by
        - w: the w component to divide by

        Returns
        - this
        """
        ...


    def div(self, x: float, y: float, z: float, w: float, dest: "Vector4f") -> "Vector4f":
        ...


    def rotate(self, quat: "Quaternionfc") -> "Vector4f":
        """
        Rotate this vector by the given quaternion `quat` and store the result in `this`.

        Arguments
        - quat: the quaternion to rotate this vector

        Returns
        - this

        See
        - Quaternionf.transform(Vector4f)
        """
        ...


    def rotate(self, quat: "Quaternionfc", dest: "Vector4f") -> "Vector4f":
        ...


    def rotateAxis(self, angle: float, x: float, y: float, z: float) -> "Vector4f":
        """
        Rotate this vector the specified radians around the given rotation axis.
        
        This vector's `w` component is ignored.
        
        If the rotation axis is either `(1, 0, 0)`, `(0, 1, 0)` or `(0, 0, 1)`,
        then .rotateX(float) rotateX(), .rotateY(float) rotateY() or
        .rotateZ(float) rotateZ(), respectively, should be used instead.

        Arguments
        - angle: the angle in radians
        - x: the x component of the rotation axis
        - y: the y component of the rotation axis
        - z: the z component of the rotation axis

        Returns
        - this
        """
        ...


    def rotateAxis(self, angle: float, aX: float, aY: float, aZ: float, dest: "Vector4f") -> "Vector4f":
        ...


    def rotateX(self, angle: float) -> "Vector4f":
        """
        Rotate this vector the specified radians around the X axis.

        Arguments
        - angle: the angle in radians

        Returns
        - this
        """
        ...


    def rotateX(self, angle: float, dest: "Vector4f") -> "Vector4f":
        ...


    def rotateY(self, angle: float) -> "Vector4f":
        """
        Rotate this vector the specified radians around the Y axis.

        Arguments
        - angle: the angle in radians

        Returns
        - this
        """
        ...


    def rotateY(self, angle: float, dest: "Vector4f") -> "Vector4f":
        ...


    def rotateZ(self, angle: float) -> "Vector4f":
        """
        Rotate this vector the specified radians around the Z axis.

        Arguments
        - angle: the angle in radians

        Returns
        - this
        """
        ...


    def rotateZ(self, angle: float, dest: "Vector4f") -> "Vector4f":
        ...


    def lengthSquared(self) -> float:
        ...


    @staticmethod
    def lengthSquared(x: float, y: float, z: float, w: float) -> float:
        """
        Get the length squared of a 4-dimensional single-precision vector.

    Author(s)
        - F. Neurath

        Arguments
        - x: the vector's x component
        - y: the vector's y component
        - z: the vector's z component
        - w: the vector's w component

        Returns
        - the length squared of the given vector
        """
        ...


    @staticmethod
    def lengthSquared(x: int, y: int, z: int, w: int) -> float:
        """
        Get the length squared of a 4-dimensional int vector.

        Arguments
        - x: the vector's x component
        - y: the vector's y component
        - z: the vector's z component
        - w: the vector's w component

        Returns
        - the length squared of the given vector
        """
        ...


    def length(self) -> float:
        ...


    @staticmethod
    def length(x: float, y: float, z: float, w: float) -> float:
        """
        Get the length of a 4-dimensional single-precision vector.

    Author(s)
        - F. Neurath

        Arguments
        - x: The vector's x component
        - y: The vector's y component
        - z: The vector's z component
        - w: The vector's w component

        Returns
        - the length of the given vector
        """
        ...


    def normalize(self) -> "Vector4f":
        """
        Normalizes this vector.

        Returns
        - this
        """
        ...


    def normalize(self, dest: "Vector4f") -> "Vector4f":
        ...


    def normalize(self, length: float) -> "Vector4f":
        """
        Scale this vector to have the given length.

        Arguments
        - length: the desired length

        Returns
        - this
        """
        ...


    def normalize(self, length: float, dest: "Vector4f") -> "Vector4f":
        ...


    def normalize3(self) -> "Vector4f":
        """
        Normalize this vector by computing only the norm of `(x, y, z)`.

        Returns
        - this
        """
        ...


    def normalize3(self, dest: "Vector4f") -> "Vector4f":
        ...


    def distance(self, v: "Vector4fc") -> float:
        ...


    def distance(self, x: float, y: float, z: float, w: float) -> float:
        ...


    def distanceSquared(self, v: "Vector4fc") -> float:
        ...


    def distanceSquared(self, x: float, y: float, z: float, w: float) -> float:
        ...


    @staticmethod
    def distance(x1: float, y1: float, z1: float, w1: float, x2: float, y2: float, z2: float, w2: float) -> float:
        """
        Return the distance between `(x1, y1, z1, w1)` and `(x2, y2, z2, w2)`.

        Arguments
        - x1: the x component of the first vector
        - y1: the y component of the first vector
        - z1: the z component of the first vector
        - w1: the w component of the first vector
        - x2: the x component of the second vector
        - y2: the y component of the second vector
        - z2: the z component of the second vector
        - w2: the 2 component of the second vector

        Returns
        - the euclidean distance
        """
        ...


    @staticmethod
    def distanceSquared(x1: float, y1: float, z1: float, w1: float, x2: float, y2: float, z2: float, w2: float) -> float:
        """
        Return the squared distance between `(x1, y1, z1, w1)` and `(x2, y2, z2, w2)`.

        Arguments
        - x1: the x component of the first vector
        - y1: the y component of the first vector
        - z1: the z component of the first vector
        - w1: the w component of the first vector
        - x2: the x component of the second vector
        - y2: the y component of the second vector
        - z2: the z component of the second vector
        - w2: the w component of the second vector

        Returns
        - the euclidean distance squared
        """
        ...


    def dot(self, v: "Vector4fc") -> float:
        ...


    def dot(self, x: float, y: float, z: float, w: float) -> float:
        ...


    def angleCos(self, v: "Vector4fc") -> float:
        ...


    def angle(self, v: "Vector4fc") -> float:
        ...


    def zero(self) -> "Vector4f":
        """
        Set all components to zero.

        Returns
        - this
        """
        ...


    def negate(self) -> "Vector4f":
        """
        Negate this vector.

        Returns
        - this
        """
        ...


    def negate(self, dest: "Vector4f") -> "Vector4f":
        ...


    def toString(self) -> str:
        """
        Return a string representation of this vector.
        
        This method creates a new DecimalFormat on every invocation with the format string "`0.000E0;-`".

        Returns
        - the string representation
        """
        ...


    def toString(self, formatter: "NumberFormat") -> str:
        """
        Return a string representation of this vector by formatting the vector components with the given NumberFormat.

        Arguments
        - formatter: the NumberFormat used to format the vector components with

        Returns
        - the string representation
        """
        ...


    def writeExternal(self, out: "ObjectOutput") -> None:
        ...


    def readExternal(self, in: "ObjectInput") -> None:
        ...


    def min(self, v: "Vector4fc") -> "Vector4f":
        """
        Set the components of this vector to be the component-wise minimum of this and the other vector.

        Arguments
        - v: the other vector

        Returns
        - this
        """
        ...


    def min(self, v: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def max(self, v: "Vector4fc") -> "Vector4f":
        """
        Set the components of this vector to be the component-wise maximum of this and the other vector.

        Arguments
        - v: the other vector

        Returns
        - this
        """
        ...


    def max(self, v: "Vector4fc", dest: "Vector4f") -> "Vector4f":
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def equals(self, v: "Vector4fc", delta: float) -> bool:
        ...


    def equals(self, x: float, y: float, z: float, w: float) -> bool:
        ...


    def smoothStep(self, v: "Vector4fc", t: float, dest: "Vector4f") -> "Vector4f":
        ...


    def hermite(self, t0: "Vector4fc", v1: "Vector4fc", t1: "Vector4fc", t: float, dest: "Vector4f") -> "Vector4f":
        ...


    def lerp(self, other: "Vector4fc", t: float) -> "Vector4f":
        """
        Linearly interpolate `this` and `other` using the given interpolation factor `t`
        and store the result in `this`.
        
        If `t` is `0.0` then the result is `this`. If the interpolation factor is `1.0`
        then the result is `other`.

        Arguments
        - other: the other vector
        - t: the interpolation factor between 0.0 and 1.0

        Returns
        - this
        """
        ...


    def lerp(self, other: "Vector4fc", t: float, dest: "Vector4f") -> "Vector4f":
        ...


    def get(self, component: int) -> float:
        ...


    def get(self, mode: int, dest: "Vector4i") -> "Vector4i":
        ...


    def get(self, dest: "Vector4f") -> "Vector4f":
        ...


    def get(self, dest: "Vector4d") -> "Vector4d":
        ...


    def maxComponent(self) -> int:
        ...


    def minComponent(self) -> int:
        ...


    def floor(self) -> "Vector4f":
        """
        Set each component of this vector to the largest (closest to positive
        infinity) `float` value that is less than or equal to that
        component and is equal to a mathematical integer.

        Returns
        - this
        """
        ...


    def floor(self, dest: "Vector4f") -> "Vector4f":
        ...


    def ceil(self) -> "Vector4f":
        """
        Set each component of this vector to the smallest (closest to negative
        infinity) `float` value that is greater than or equal to that
        component and is equal to a mathematical integer.

        Returns
        - this
        """
        ...


    def ceil(self, dest: "Vector4f") -> "Vector4f":
        ...


    def round(self) -> "Vector4f":
        """
        Set each component of this vector to the closest float that is equal to
        a mathematical integer, with ties rounding to positive infinity.

        Returns
        - this
        """
        ...


    def round(self, dest: "Vector4f") -> "Vector4f":
        ...


    def isFinite(self) -> bool:
        ...


    def absolute(self) -> "Vector4f":
        """
        Compute the absolute of each of this vector's components.

        Returns
        - this
        """
        ...


    def absolute(self, dest: "Vector4f") -> "Vector4f":
        ...


    def clone(self) -> "Object":
        ...



"""
Python module generated from Java source file org.bukkit.RegionAccessor

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Random
from java.util.function import Consumer
from java.util.function import Predicate
from org.bukkit import *
from org.bukkit.block import Biome
from org.bukkit.block import BlockState
from org.bukkit.block.data import BlockData
from org.bukkit.entity import Entity
from org.bukkit.entity import EntityType
from org.bukkit.entity import LivingEntity
from typing import Any, Callable, Iterable, Tuple


class RegionAccessor:
    """
    A RegionAccessor gives access to getting, modifying and spawning Biome, BlockState and Entity,
    as well as generating some basic structures.
    """

    def getBiome(self, location: "Location") -> "Biome":
        """
        Gets the Biome at the given Location.

        Arguments
        - location: the location of the biome

        Returns
        - Biome at the given location
        """
        ...


    def getBiome(self, x: int, y: int, z: int) -> "Biome":
        """
        Gets the Biome at the given coordinates.

        Arguments
        - x: X-coordinate of the block
        - y: Y-coordinate of the block
        - z: Z-coordinate of the block

        Returns
        - Biome at the given coordinates
        """
        ...


    def setBiome(self, location: "Location", biome: "Biome") -> None:
        """
        Sets the Biome at the given Location.

        Arguments
        - location: the location of the biome
        - biome: New Biome type for this block
        """
        ...


    def setBiome(self, x: int, y: int, z: int, biome: "Biome") -> None:
        """
        Sets the Biome for the given block coordinates

        Arguments
        - x: X-coordinate of the block
        - y: Y-coordinate of the block
        - z: Z-coordinate of the block
        - biome: New Biome type for this block
        """
        ...


    def getBlockState(self, location: "Location") -> "BlockState":
        """
        Gets the BlockState at the given Location.

        Arguments
        - location: The location of the block state

        Returns
        - Block state at the given location
        """
        ...


    def getBlockState(self, x: int, y: int, z: int) -> "BlockState":
        """
        Gets the BlockState at the given coordinates.

        Arguments
        - x: X-coordinate of the block state
        - y: Y-coordinate of the block state
        - z: Z-coordinate of the block state

        Returns
        - Block state at the given coordinates
        """
        ...


    def getBlockData(self, location: "Location") -> "BlockData":
        """
        Gets the BlockData at the given Location.

        Arguments
        - location: The location of the block data

        Returns
        - Block data at the given location
        """
        ...


    def getBlockData(self, x: int, y: int, z: int) -> "BlockData":
        """
        Gets the BlockData at the given coordinates.

        Arguments
        - x: X-coordinate of the block data
        - y: Y-coordinate of the block data
        - z: Z-coordinate of the block data

        Returns
        - Block data at the given coordinates
        """
        ...


    def getType(self, location: "Location") -> "Material":
        """
        Gets the type of the block at the given Location.

        Arguments
        - location: The location of the block

        Returns
        - Material at the given coordinates
        """
        ...


    def getType(self, x: int, y: int, z: int) -> "Material":
        """
        Gets the type of the block at the given coordinates.

        Arguments
        - x: X-coordinate of the block
        - y: Y-coordinate of the block
        - z: Z-coordinate of the block

        Returns
        - Material at the given coordinates
        """
        ...


    def setBlockData(self, location: "Location", blockData: "BlockData") -> None:
        """
        Sets the BlockData at the given Location.

        Arguments
        - location: The location of the block
        - blockData: The block data to set the block to
        """
        ...


    def setBlockData(self, x: int, y: int, z: int, blockData: "BlockData") -> None:
        """
        Sets the BlockData at the given coordinates.

        Arguments
        - x: X-coordinate of the block
        - y: Y-coordinate of the block
        - z: Z-coordinate of the block
        - blockData: The block data to set the block to
        """
        ...


    def setType(self, location: "Location", material: "Material") -> None:
        """
        Sets the Material at the given Location.

        Arguments
        - location: The location of the block
        - material: The type to set the block to
        """
        ...


    def setType(self, x: int, y: int, z: int, material: "Material") -> None:
        """
        Sets the Material at the given coordinates.

        Arguments
        - x: X-coordinate of the block
        - y: Y-coordinate of the block
        - z: Z-coordinate of the block
        - material: The type to set the block to
        """
        ...


    def generateTree(self, location: "Location", random: "Random", type: "TreeType") -> bool:
        """
        Creates a tree at the given Location

        Arguments
        - location: Location to spawn the tree
        - random: Random to use to generated the tree
        - type: Type of the tree to create

        Returns
        - True if the tree was created successfully, otherwise False
        """
        ...


    def generateTree(self, location: "Location", random: "Random", type: "TreeType", stateConsumer: "Consumer"["BlockState"]) -> bool:
        """
        Creates a tree at the given Location
        
        The provided consumer gets called for every block which gets changed
        as a result of the tree generation. When the consumer gets called no
        modifications to the world are done yet. Which means, that calling
        .getBlockState(Location) in the consumer while return the state
        of the block before the generation.
        
        Modifications done to the BlockState in the consumer are respected,
        which means that it is not necessary to call BlockState.update()

        Arguments
        - location: Location to spawn the tree
        - random: Random to use to generated the tree
        - type: Type of the tree to create
        - stateConsumer: The consumer which should get called for every block which gets changed

        Returns
        - True if the tree was created successfully, otherwise False
        """
        ...


    def generateTree(self, location: "Location", random: "Random", type: "TreeType", statePredicate: "Predicate"["BlockState"]) -> bool:
        """
        Creates a tree at the given Location
        
        The provided predicate gets called for every block which gets changed
        as a result of the tree generation. When the predicate gets called no
        modifications to the world are done yet. Which means, that calling
        .getBlockState(Location) in the predicate will return the state
        of the block before the generation.
        
        If the predicate returns `True` the block gets set in the world.
        If it returns `False` the block won't get set in the world.

        Arguments
        - location: Location to spawn the tree
        - random: Random to use to generated the tree
        - type: Type of the tree to create
        - statePredicate: The predicate which should get used to test if a block should be set or not.

        Returns
        - True if the tree was created successfully, otherwise False
        """
        ...


    def spawnEntity(self, location: "Location", type: "EntityType") -> "Entity":
        """
        Creates a entity at the given Location

        Arguments
        - location: The location to spawn the entity
        - type: The entity to spawn

        Returns
        - Resulting Entity of this method
        """
        ...


    def spawnEntity(self, loc: "Location", type: "EntityType", randomizeData: bool) -> "Entity":
        """
        Creates a new entity at the given Location.

        Arguments
        - loc: the location at which the entity will be spawned.
        - type: the entity type that determines the entity to spawn.
        - randomizeData: whether or not the entity's data should be randomised
                             before spawning. By default entities are randomised
                             before spawning in regards to their equipment, age,
                             attributes, etc.
                             An example of this randomization would be the color of
                             a sheep, random enchantments on the equipment of mobs
                             or even a zombie becoming a chicken jockey.
                             If set to False, the entity will not be randomised
                             before spawning, meaning all their data will remain
                             in their default state and not further modifications
                             to the entity will be made.
                             Notably only entities that extend the
                             org.bukkit.entity.Mob interface provide
                             randomisation logic for their spawn.
                             This parameter is hence useless for any other type
                             of entity.

        Returns
        - the spawned entity instance.
        """
        ...


    def getEntities(self) -> list["Entity"]:
        """
        Get a list of all entities in this RegionAccessor

        Returns
        - A List of all Entities currently residing in this world accessor
        """
        ...


    def getLivingEntities(self) -> list["LivingEntity"]:
        """
        Get a list of all living entities in this RegionAccessor

        Returns
        - A List of all LivingEntities currently residing in this world accessor
        """
        ...


    def getEntitiesByClass(self, cls: type["T"]) -> Iterable["T"]:
        """
        Get a collection of all entities in this RegionAccessor matching the given
        class/interface
        
        Type `<T>`: an entity subclass

        Arguments
        - cls: The class representing the type of entity to match

        Returns
        - A List of all Entities currently residing in this world accessor
            that match the given class/interface
        """
        ...


    def getEntitiesByClasses(self, *classes: Tuple[type[Any], ...]) -> Iterable["Entity"]:
        """
        Get a collection of all entities in this RegionAccessor matching any of the
        given classes/interfaces

        Arguments
        - classes: The classes representing the types of entity to match

        Returns
        - A List of all Entities currently residing in this world accessor
            that match one or more of the given classes/interfaces
        """
        ...


    def createEntity(self, location: "Location", clazz: type["T"]) -> "T":
        """
        Creates an entity of a specific class at the given Location but
        does not spawn it in the world.
        
        **Note:** The created entity keeps a reference to the world it was
        created in, care should be taken that the entity does not outlive the
        world instance as this will lead to memory leaks.
        
        Type `<T>`: the class of the Entity to create

        Arguments
        - location: the Location to create the entity at
        - clazz: the class of the Entity to spawn

        Returns
        - an instance of the created Entity

        See
        - Entity.createSnapshot()
        """
        ...


    def spawn(self, location: "Location", clazz: type["T"]) -> "T":
        """
        Spawn an entity of a specific class at the given Location
        
        Type `<T>`: the class of the Entity to spawn

        Arguments
        - location: the Location to spawn the entity at
        - clazz: the class of the Entity to spawn

        Returns
        - an instance of the spawned Entity

        Raises
        - IllegalArgumentException: if either parameter is null or the
            Entity requested cannot be spawned
        """
        ...


    def spawn(self, location: "Location", clazz: type["T"], function: "Consumer"["T"]) -> "T":
        """
        Spawn an entity of a specific class at the given Location, with
        the supplied function run before the entity is added to the world.
        
        Note that when the function is run, the entity will not be actually in
        the world. Any operation involving such as teleporting the entity is undefined
        until after this function returns.
        
        Type `<T>`: the class of the Entity to spawn

        Arguments
        - location: the Location to spawn the entity at
        - clazz: the class of the Entity to spawn
        - function: the function to be run before the entity is spawned.

        Returns
        - an instance of the spawned Entity

        Raises
        - IllegalArgumentException: if either parameter is null or the
            Entity requested cannot be spawned
        """
        ...


    def spawn(self, location: "Location", clazz: type["T"], randomizeData: bool, function: "Consumer"["T"]) -> "T":
        """
        Creates a new entity at the given Location with the supplied
        function run before the entity is added to the world.
        
        Note that when the function is run, the entity will not be actually in
        the world. Any operation involving such as teleporting the entity is undefined
        until after this function returns.
        The passed function however is run after the potential entity's spawn
        randomization and hence already allows access to the values of the mob,
        whether or not those were randomized, such as attributes or the entity
        equipment.
        
        Type `<T>`: the generic type of the entity that is being created.

        Arguments
        - location: the location at which the entity will be spawned.
        - clazz: the class of the Entity that is to be spawned.
        - randomizeData: whether or not the entity's data should be randomised
                             before spawning. By default entities are randomised
                             before spawning in regards to their equipment, age,
                             attributes, etc.
                             An example of this randomization would be the color of
                             a sheep, random enchantments on the equipment of mobs
                             or even a zombie becoming a chicken jockey.
                             If set to False, the entity will not be randomised
                             before spawning, meaning all their data will remain
                             in their default state and not further modifications
                             to the entity will be made.
                             Notably only entities that extend the
                             org.bukkit.entity.Mob interface provide
                             randomisation logic for their spawn.
                             This parameter is hence useless for any other type
                             of entity.
        - function: the function to be run before the entity is spawned.

        Returns
        - the spawned entity instance.

        Raises
        - IllegalArgumentException: if either the world or clazz parameter are null.
        """
        ...


    def getHighestBlockYAt(self, x: int, z: int) -> int:
        """
        Gets the highest non-empty (impassable) coordinate at the given
        coordinates.

        Arguments
        - x: X-coordinate of the blocks
        - z: Z-coordinate of the blocks

        Returns
        - Y-coordinate of the highest non-empty block
        """
        ...


    def getHighestBlockYAt(self, location: "Location") -> int:
        """
        Gets the highest non-empty (impassable) coordinate at the given
        Location.

        Arguments
        - location: Location of the blocks

        Returns
        - Y-coordinate of the highest non-empty block
        """
        ...


    def getHighestBlockYAt(self, x: int, z: int, heightMap: "HeightMap") -> int:
        """
        Gets the highest coordinate corresponding to the HeightMap at the
        given coordinates.

        Arguments
        - x: X-coordinate of the blocks
        - z: Z-coordinate of the blocks
        - heightMap: the heightMap that is used to determine the highest
        point

        Returns
        - Y-coordinate of the highest block corresponding to the
        HeightMap
        """
        ...


    def getHighestBlockYAt(self, location: "Location", heightMap: "HeightMap") -> int:
        """
        Gets the highest coordinate corresponding to the HeightMap at the
        given Location.

        Arguments
        - location: Location of the blocks
        - heightMap: the heightMap that is used to determine the highest
        point

        Returns
        - Y-coordinate of the highest block corresponding to the
        HeightMap
        """
        ...


    def addEntity(self, entity: "T") -> "T":
        """
        Spawns a previously created entity in the world. 
        The provided entity must not have already been spawned in a world.
        
        Type `<T>`: the generic type of the entity that is being added.

        Arguments
        - entity: the entity to add

        Returns
        - the entity now in the world
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.AggregateFutureState

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.util.concurrent import *
from com.google.j2objc.annotations import ReflectionSupport
from java.util.concurrent.atomic import AtomicIntegerFieldUpdater
from java.util.concurrent.atomic import AtomicReferenceFieldUpdater
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AggregateFutureState(TrustedFuture):
    """
    A helper which does some thread-safe operations for aggregate futures, which must be implemented
    differently in GWT. Namely:
    
    
      - Lazily initializes a set of seen exceptions
      - Decrements a counter atomically
    """





"""
Python module generated from Java source file com.google.common.hash.Hashing

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.hash import *
from com.google.errorprone.annotations import Immutable
from com.google.j2objc.annotations import J2ObjCIncompatible
from java.lang.invoke import MethodHandle
from java.lang.invoke import MethodHandles
from java.security import Key
from java.util import Arrays
from java.util import Collections
from java.util import Iterator
from java.util.zip import Adler32
from java.util.zip import CRC32
from java.util.zip import Checksum
from javax.annotation import CheckForNull
from javax.crypto.spec import SecretKeySpec
from typing import Any, Callable, Iterable, Tuple


class Hashing:
    """
    Static methods to obtain HashFunction instances, and other static hashing-related
    utilities.
    
    A comparison of the various hash functions can be found <a
    href="https://docs.google.com/spreadsheets/d/1_q2EVcxA2HjcrlVMbaqXwMj31h9M5-Bqj_m8vITOwwk/">here</a>.

    Author(s)
    - Kurt Alfred Kluever

    Since
    - 11.0
    """

    @staticmethod
    def goodFastHash(minimumBits: int) -> "HashFunction":
        """
        Returns a general-purpose, **temporary-use**, non-cryptographic hash function. The algorithm
        the returned function implements is unspecified and subject to change without notice.
        
        **Warning:** a new random seed for these functions is chosen each time the `Hashing` class is loaded. **Do not use this method** if hash codes may escape the current
        process in any way, for example being sent over RPC, or saved to disk. For a general-purpose,
        non-cryptographic hash function that will never change behavior, we suggest .murmur3_128.
        
        Repeated calls to this method on the same loaded `Hashing` class, using the same value
        for `minimumBits`, will return identically-behaving HashFunction instances.

        Arguments
        - minimumBits: a positive integer. This can be arbitrarily large. The returned HashFunction instance may use memory proportional to this integer.

        Returns
        - a hash function, described above, that produces hash codes of length `minimumBits` or greater
        """
        ...


    @staticmethod
    def murmur3_32(seed: int) -> "HashFunction":
        """
        Returns a hash function implementing the <a
        href="https://github.com/aappleby/smhasher/blob/master/src/MurmurHash3.cpp">32-bit murmur3
        algorithm, x86 variant</a> (little-endian variant), using the given seed value, **with a known
        bug** as described in the deprecation text.
        
        The C++ equivalent is the MurmurHash3_x86_32 function (Murmur3A), which however does not
        have the bug.

        Deprecated
        - This implementation produces incorrect hash values from the HashFunction.hashString method if the string contains non-BMP characters. Use .murmur3_32_fixed(int) instead.
        """
        ...


    @staticmethod
    def murmur3_32() -> "HashFunction":
        """
        Returns a hash function implementing the <a
        href="https://github.com/aappleby/smhasher/blob/master/src/MurmurHash3.cpp">32-bit murmur3
        algorithm, x86 variant</a> (little-endian variant), using the given seed value, **with a known
        bug** as described in the deprecation text.
        
        The C++ equivalent is the MurmurHash3_x86_32 function (Murmur3A), which however does not
        have the bug.

        Deprecated
        - This implementation produces incorrect hash values from the HashFunction.hashString method if the string contains non-BMP characters. Use .murmur3_32_fixed() instead.
        """
        ...


    @staticmethod
    def murmur3_32_fixed(seed: int) -> "HashFunction":
        """
        Returns a hash function implementing the <a
        href="https://github.com/aappleby/smhasher/blob/master/src/MurmurHash3.cpp">32-bit murmur3
        algorithm, x86 variant</a> (little-endian variant), using the given seed value.
        
        The exact C++ equivalent is the MurmurHash3_x86_32 function (Murmur3A).
        
        This method is called `murmur3_32_fixed` because it fixes a bug in the `HashFunction` returned by the original `murmur3_32` method.

        Since
        - 31.0
        """
        ...


    @staticmethod
    def murmur3_32_fixed() -> "HashFunction":
        """
        Returns a hash function implementing the <a
        href="https://github.com/aappleby/smhasher/blob/master/src/MurmurHash3.cpp">32-bit murmur3
        algorithm, x86 variant</a> (little-endian variant), using a seed value of zero.
        
        The exact C++ equivalent is the MurmurHash3_x86_32 function (Murmur3A).
        
        This method is called `murmur3_32_fixed` because it fixes a bug in the `HashFunction` returned by the original `murmur3_32` method.

        Since
        - 31.0
        """
        ...


    @staticmethod
    def murmur3_128(seed: int) -> "HashFunction":
        """
        Returns a hash function implementing the <a
        href="https://github.com/aappleby/smhasher/blob/master/src/MurmurHash3.cpp">128-bit murmur3
        algorithm, x64 variant</a> (little-endian variant), using the given seed value.
        
        The exact C++ equivalent is the MurmurHash3_x64_128 function (Murmur3F).
        """
        ...


    @staticmethod
    def murmur3_128() -> "HashFunction":
        """
        Returns a hash function implementing the <a
        href="https://github.com/aappleby/smhasher/blob/master/src/MurmurHash3.cpp">128-bit murmur3
        algorithm, x64 variant</a> (little-endian variant), using a seed value of zero.
        
        The exact C++ equivalent is the MurmurHash3_x64_128 function (Murmur3F).
        """
        ...


    @staticmethod
    def sipHash24() -> "HashFunction":
        """
        Returns a hash function implementing the <a href="https://131002.net/siphash/">64-bit
        SipHash-2-4 algorithm</a> using a seed value of `k = 00 01 02 ...`.

        Since
        - 15.0
        """
        ...


    @staticmethod
    def sipHash24(k0: int, k1: int) -> "HashFunction":
        """
        Returns a hash function implementing the <a href="https://131002.net/siphash/">64-bit
        SipHash-2-4 algorithm</a> using the given seed.

        Since
        - 15.0
        """
        ...


    @staticmethod
    def md5() -> "HashFunction":
        """
        Returns a hash function implementing the MD5 hash algorithm (128 hash bits).

        Deprecated
        - If you must interoperate with a system that requires MD5, then use this method,
            despite its deprecation. But if you can choose your hash function, avoid MD5, which is
            neither fast nor secure. As of January 2017, we suggest:
            
              - For security:
                  Hashing.sha256 or a higher-level API.
              - For speed: Hashing.goodFastHash, though see its docs for caveats.
            
        """
        ...


    @staticmethod
    def sha1() -> "HashFunction":
        """
        Returns a hash function implementing the SHA-1 algorithm (160 hash bits).

        Deprecated
        - If you must interoperate with a system that requires SHA-1, then use this method,
            despite its deprecation. But if you can choose your hash function, avoid SHA-1, which is
            neither fast nor secure. As of January 2017, we suggest:
            
              - For security:
                  Hashing.sha256 or a higher-level API.
              - For speed: Hashing.goodFastHash, though see its docs for caveats.
            
        """
        ...


    @staticmethod
    def sha256() -> "HashFunction":
        """
        Returns a hash function implementing the SHA-256 algorithm (256 hash bits).
        """
        ...


    @staticmethod
    def sha384() -> "HashFunction":
        """
        Returns a hash function implementing the SHA-384 algorithm (384 hash bits).

        Since
        - 19.0
        """
        ...


    @staticmethod
    def sha512() -> "HashFunction":
        """
        Returns a hash function implementing the SHA-512 algorithm (512 hash bits).
        """
        ...


    @staticmethod
    def hmacMd5(key: "Key") -> "HashFunction":
        """
        Returns a hash function implementing the Message Authentication Code (MAC) algorithm, using the
        MD5 (128 hash bits) hash function and the given secret key.
        
        If you are designing a new system that needs HMAC, prefer .hmacSha256 or other
        future-proof algorithms <a
        href="https://datatracker.ietf.org/doc/html/rfc6151#section-2.3">over `hmacMd5`</a>.

        Arguments
        - key: the secret key

        Raises
        - IllegalArgumentException: if the given key is inappropriate for initializing this MAC

        Since
        - 20.0
        """
        ...


    @staticmethod
    def hmacMd5(key: list[int]) -> "HashFunction":
        """
        Returns a hash function implementing the Message Authentication Code (MAC) algorithm, using the
        MD5 (128 hash bits) hash function and a SecretKeySpec created from the given byte array
        and the MD5 algorithm.
        
        If you are designing a new system that needs HMAC, prefer .hmacSha256 or other
        future-proof algorithms <a
        href="https://datatracker.ietf.org/doc/html/rfc6151#section-2.3">over `hmacMd5`</a>.

        Arguments
        - key: the key material of the secret key

        Since
        - 20.0
        """
        ...


    @staticmethod
    def hmacSha1(key: "Key") -> "HashFunction":
        """
        Returns a hash function implementing the Message Authentication Code (MAC) algorithm, using the
        SHA-1 (160 hash bits) hash function and the given secret key.

        Arguments
        - key: the secret key

        Raises
        - IllegalArgumentException: if the given key is inappropriate for initializing this MAC

        Since
        - 20.0
        """
        ...


    @staticmethod
    def hmacSha1(key: list[int]) -> "HashFunction":
        """
        Returns a hash function implementing the Message Authentication Code (MAC) algorithm, using the
        SHA-1 (160 hash bits) hash function and a SecretKeySpec created from the given byte
        array and the SHA-1 algorithm.

        Arguments
        - key: the key material of the secret key

        Since
        - 20.0
        """
        ...


    @staticmethod
    def hmacSha256(key: "Key") -> "HashFunction":
        """
        Returns a hash function implementing the Message Authentication Code (MAC) algorithm, using the
        SHA-256 (256 hash bits) hash function and the given secret key.

        Arguments
        - key: the secret key

        Raises
        - IllegalArgumentException: if the given key is inappropriate for initializing this MAC

        Since
        - 20.0
        """
        ...


    @staticmethod
    def hmacSha256(key: list[int]) -> "HashFunction":
        """
        Returns a hash function implementing the Message Authentication Code (MAC) algorithm, using the
        SHA-256 (256 hash bits) hash function and a SecretKeySpec created from the given byte
        array and the SHA-256 algorithm.

        Arguments
        - key: the key material of the secret key

        Since
        - 20.0
        """
        ...


    @staticmethod
    def hmacSha512(key: "Key") -> "HashFunction":
        """
        Returns a hash function implementing the Message Authentication Code (MAC) algorithm, using the
        SHA-512 (512 hash bits) hash function and the given secret key.

        Arguments
        - key: the secret key

        Raises
        - IllegalArgumentException: if the given key is inappropriate for initializing this MAC

        Since
        - 20.0
        """
        ...


    @staticmethod
    def hmacSha512(key: list[int]) -> "HashFunction":
        """
        Returns a hash function implementing the Message Authentication Code (MAC) algorithm, using the
        SHA-512 (512 hash bits) hash function and a SecretKeySpec created from the given byte
        array and the SHA-512 algorithm.

        Arguments
        - key: the key material of the secret key

        Since
        - 20.0
        """
        ...


    @staticmethod
    def crc32c() -> "HashFunction":
        """
        Returns a hash function implementing the CRC32C checksum algorithm (32 hash bits) as described
        by RFC 3720, Section 12.1.
        
        This function is best understood as a <a
        href="https://en.wikipedia.org/wiki/Checksum">checksum</a> rather than a True <a
        href="https://en.wikipedia.org/wiki/Hash_function">hash function</a>.

        Since
        - 18.0
        """
        ...


    @staticmethod
    def crc32() -> "HashFunction":
        """
        Returns a hash function implementing the CRC-32 checksum algorithm (32 hash bits).
        
        To get the `long` value equivalent to Checksum.getValue() for a `HashCode` produced by this function, use HashCode.padToLong().
        
        This function is best understood as a <a
        href="https://en.wikipedia.org/wiki/Checksum">checksum</a> rather than a True <a
        href="https://en.wikipedia.org/wiki/Hash_function">hash function</a>.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def adler32() -> "HashFunction":
        """
        Returns a hash function implementing the Adler-32 checksum algorithm (32 hash bits).
        
        To get the `long` value equivalent to Checksum.getValue() for a `HashCode` produced by this function, use HashCode.padToLong().
        
        This function is best understood as a <a
        href="https://en.wikipedia.org/wiki/Checksum">checksum</a> rather than a True <a
        href="https://en.wikipedia.org/wiki/Hash_function">hash function</a>.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def farmHashFingerprint64() -> "HashFunction":
        """
        Returns a hash function implementing FarmHash's Fingerprint64, an open-source algorithm.
        
        This is designed for generating persistent fingerprints of strings. It isn't
        cryptographically secure, but it produces a high-quality hash with fewer collisions than some
        alternatives we've used in the past.
        
        FarmHash fingerprints are encoded by HashCode.asBytes in little-endian order. This
        means HashCode.asLong is guaranteed to return the same value that
        farmhash::Fingerprint64() would for the same input (when compared using com.google.common.primitives.UnsignedLongs's encoding of 64-bit unsigned numbers).
        
        This function is best understood as a <a
        href="https://en.wikipedia.org/wiki/Fingerprint_(computing)">fingerprint</a> rather than a True
        <a href="https://en.wikipedia.org/wiki/Hash_function">hash function</a>.

        Since
        - 20.0
        """
        ...


    @staticmethod
    def fingerprint2011() -> "HashFunction":
        """
        Returns a hash function implementing the Fingerprint2011 hashing function (64 hash bits).
        
        This is designed for generating persistent fingerprints of strings. It isn't
        cryptographically secure, but it produces a high-quality hash with few collisions. Fingerprints
        generated using this are byte-wise identical to those created using the C++ version, but note
        that this uses unsigned integers (see com.google.common.primitives.UnsignedInts).
        Comparisons between the two should take this into account.
        
        Fingerprint2011() is a form of Murmur2 on strings up to 32 bytes and a form of CityHash for
        longer strings. It could have been one or the other throughout. The main advantage of the
        combination is that CityHash has a bunch of special cases for short strings that don't need to
        be replicated here. The result will never be 0 or 1.
        
        This function is best understood as a <a
        href="https://en.wikipedia.org/wiki/Fingerprint_(computing)">fingerprint</a> rather than a True
        <a href="https://en.wikipedia.org/wiki/Hash_function">hash function</a>.

        Since
        - 31.1
        """
        ...


    @staticmethod
    def consistentHash(hashCode: "HashCode", buckets: int) -> int:
        """
        Assigns to `hashCode` a "bucket" in the range `[0, buckets)`, in a uniform manner
        that minimizes the need for remapping as `buckets` grows. That is, `consistentHash(h, n)` equals:
        
        
          - `n - 1`, with approximate probability `1/n`
          - `consistentHash(h, n - 1)`, otherwise (probability `1 - 1/n`)
        
        
        This method is suitable for the common use case of dividing work among buckets that meet the
        following conditions:
        
        
          - You want to assign the same fraction of inputs to each bucket.
          - When you reduce the number of buckets, you can accept that the most recently added
              buckets will be removed first. More concretely, if you are dividing traffic among tasks,
              you can decrease the number of tasks from 15 and 10, killing off the final 5 tasks, and
              `consistentHash` will handle it. If, however, you are dividing traffic among
              servers `alpha`, `bravo`, and `charlie` and you occasionally need to
              take each of the servers offline, `consistentHash` will be a poor fit: It provides
              no way for you to specify which of the three buckets is disappearing. Thus, if your
              buckets change from `[alpha, bravo, charlie]` to `[bravo, charlie]`, it will
              assign all the old `alpha` traffic to `bravo` and all the old `bravo`
              traffic to `charlie`, rather than letting `bravo` keep its traffic.
        
        
        See the <a href="http://en.wikipedia.org/wiki/Consistent_hashing">Wikipedia article on
        consistent hashing</a> for more information.
        """
        ...


    @staticmethod
    def consistentHash(input: int, buckets: int) -> int:
        """
        Assigns to `input` a "bucket" in the range `[0, buckets)`, in a uniform manner that
        minimizes the need for remapping as `buckets` grows. That is, `consistentHash(h,
        n)` equals:
        
        
          - `n - 1`, with approximate probability `1/n`
          - `consistentHash(h, n - 1)`, otherwise (probability `1 - 1/n`)
        
        
        This method is suitable for the common use case of dividing work among buckets that meet the
        following conditions:
        
        
          - You want to assign the same fraction of inputs to each bucket.
          - When you reduce the number of buckets, you can accept that the most recently added
              buckets will be removed first. More concretely, if you are dividing traffic among tasks,
              you can decrease the number of tasks from 15 and 10, killing off the final 5 tasks, and
              `consistentHash` will handle it. If, however, you are dividing traffic among
              servers `alpha`, `bravo`, and `charlie` and you occasionally need to
              take each of the servers offline, `consistentHash` will be a poor fit: It provides
              no way for you to specify which of the three buckets is disappearing. Thus, if your
              buckets change from `[alpha, bravo, charlie]` to `[bravo, charlie]`, it will
              assign all the old `alpha` traffic to `bravo` and all the old `bravo`
              traffic to `charlie`, rather than letting `bravo` keep its traffic.
        
        
        See the <a href="http://en.wikipedia.org/wiki/Consistent_hashing">Wikipedia article on
        consistent hashing</a> for more information.
        """
        ...


    @staticmethod
    def combineOrdered(hashCodes: Iterable["HashCode"]) -> "HashCode":
        """
        Returns a hash code, having the same bit length as each of the input hash codes, that combines
        the information of these hash codes in an ordered fashion. That is, whenever two equal hash
        codes are produced by two calls to this method, it is *as likely as possible* that each
        was computed from the *same* input hash codes in the *same* order.

        Raises
        - IllegalArgumentException: if `hashCodes` is empty, or the hash codes do not all
            have the same bit length
        """
        ...


    @staticmethod
    def combineUnordered(hashCodes: Iterable["HashCode"]) -> "HashCode":
        """
        Returns a hash code, having the same bit length as each of the input hash codes, that combines
        the information of these hash codes in an unordered fashion. That is, whenever two equal hash
        codes are produced by two calls to this method, it is *as likely as possible* that each
        was computed from the *same* input hash codes in *some* order.

        Raises
        - IllegalArgumentException: if `hashCodes` is empty, or the hash codes do not all
            have the same bit length
        """
        ...


    @staticmethod
    def concatenating(first: "HashFunction", second: "HashFunction", *rest: Tuple["HashFunction", ...]) -> "HashFunction":
        """
        Returns a hash function which computes its hash code by concatenating the hash codes of the
        underlying hash functions together. This can be useful if you need to generate hash codes of a
        specific length.
        
        For example, if you need 1024-bit hash codes, you could join two Hashing.sha512 hash
        functions together: `Hashing.concatenating(Hashing.sha512(), Hashing.sha512())`.

        Since
        - 19.0
        """
        ...


    @staticmethod
    def concatenating(hashFunctions: Iterable["HashFunction"]) -> "HashFunction":
        """
        Returns a hash function which computes its hash code by concatenating the hash codes of the
        underlying hash functions together. This can be useful if you need to generate hash codes of a
        specific length.
        
        For example, if you need 1024-bit hash codes, you could join two Hashing.sha512 hash
        functions together: `Hashing.concatenating(Hashing.sha512(), Hashing.sha512())`.

        Since
        - 19.0
        """
        ...



"""
Python module generated from Java source file java.net.InetSocketAddress

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import ObjectStreamException
from java.io import ObjectStreamField
from java.net import *
from typing import Any, Callable, Iterable, Tuple


class InetSocketAddress(SocketAddress):
    """
    This class implements an IP Socket Address (IP address + port number)
    It can also be a pair (hostname + port number), in which case an attempt
    will be made to resolve the hostname. If resolution fails then the address
    is said to be <I>unresolved</I> but can still be used on some circumstances
    like connecting through a proxy.
    
    It provides an immutable object used by sockets for binding, connecting, or
    as returned values.
    
    The *wildcard* is a special local IP address. It usually means "any"
    and can only be used for `bind` operations.

    See
    - java.net.ServerSocket

    Since
    - 1.4
    """

    def __init__(self, port: int):
        """
        Creates a socket address where the IP address is the wildcard address
        and the port number a specified value.
        
        A valid port value is between 0 and 65535.
        A port number of `zero` will let the system pick up an
        ephemeral port in a `bind` operation.

        Arguments
        - port: The port number

        Raises
        - IllegalArgumentException: if the port parameter is outside the specified
        range of valid port values.
        """
        ...


    def __init__(self, addr: "InetAddress", port: int):
        """
        Creates a socket address from an IP address and a port number.
        
        A valid port value is between 0 and 65535.
        A port number of `zero` will let the system pick up an
        ephemeral port in a `bind` operation.
        <P>
        A `null` address will assign the *wildcard* address.

        Arguments
        - addr: The IP address
        - port: The port number

        Raises
        - IllegalArgumentException: if the port parameter is outside the specified
        range of valid port values.
        """
        ...


    def __init__(self, hostname: str, port: int):
        """
        Creates a socket address from a hostname and a port number.
        
        An attempt will be made to resolve the hostname into an InetAddress.
        If that attempt fails, the address will be flagged as <I>unresolved</I>.
        
        If there is a security manager, its `checkConnect` method
        is called with the host name as its argument to check the permission
        to resolve it. This could result in a SecurityException.
        <P>
        A valid port value is between 0 and 65535.
        A port number of `zero` will let the system pick up an
        ephemeral port in a `bind` operation.

        Arguments
        - hostname: the Host name
        - port: The port number

        Raises
        - IllegalArgumentException: if the port parameter is outside the range
        of valid port values, or if the hostname parameter is `null`.
        - SecurityException: if a security manager is present and
                                  permission to resolve the host name is
                                  denied.

        See
        - .isUnresolved()
        """
        ...


    @staticmethod
    def createUnresolved(host: str, port: int) -> "InetSocketAddress":
        """
        Creates an unresolved socket address from a hostname and a port number.
        
        No attempt will be made to resolve the hostname into an InetAddress.
        The address will be flagged as <I>unresolved</I>.
        
        A valid port value is between 0 and 65535.
        A port number of `zero` will let the system pick up an
        ephemeral port in a `bind` operation.

        Arguments
        - host: the Host name
        - port: The port number

        Returns
        - an `InetSocketAddress` representing the unresolved
                 socket address

        Raises
        - IllegalArgumentException: if the port parameter is outside
                         the range of valid port values, or if the hostname
                         parameter is `null`.

        See
        - .isUnresolved()

        Since
        - 1.5
        """
        ...


    def getPort(self) -> int:
        """
        Gets the port number.

        Returns
        - the port number.
        """
        ...


    def getAddress(self) -> "InetAddress":
        """
        Gets the `InetAddress`.

        Returns
        - the InetAddress or `null` if it is unresolved.
        """
        ...


    def getHostName(self) -> str:
        """
        Gets the `hostname`.
        Note: This method may trigger a name service reverse lookup if the
        address was created with a literal IP address.

        Returns
        - the hostname part of the address.
        """
        ...


    def getHostString(self) -> str:
        """
        Returns the hostname, or the String form of the address if it
        doesn't have a hostname (it was created using a literal).
        This has the benefit of **not** attempting a reverse lookup.

        Returns
        - the hostname, or String representation of the address.

        Since
        - 1.7
        """
        ...


    def isUnresolved(self) -> bool:
        """
        Checks whether the address has been resolved or not.

        Returns
        - `True` if the hostname couldn't be resolved into
                 an `InetAddress`.
        """
        ...


    def toString(self) -> str:
        """
        Constructs a string representation of this InetSocketAddress.
        This string is constructed by calling InetAddress.toString()
        on the InetAddress and concatenating the port number (with a colon).
        
        If the address is an IPv6 address, the IPv6 literal is enclosed in
        square brackets, for example: `"localhost/[0:0:0:0:0:0:0:1]:80"`.
        If the address is .isUnresolved() unresolved,
        `<unresolved>` is displayed in place of the address literal, for
        example `"foo/<unresolved>:80"`.
        
        To retrieve a string representation of the hostname or the address, use
        .getHostString(), rather than parsing the string returned by this
        .toString() method.

        Returns
        - a string representation of this object.
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Compares this object against the specified object.
        The result is `True` if and only if the argument is
        not `null` and it represents the same address as
        this object.
        
        Two instances of `InetSocketAddress` represent the same
        address if both the InetAddresses (or hostnames if it is unresolved) and port
        numbers are equal.
        If both addresses are unresolved, then the hostname and the port number
        are compared.
        
        Note: Hostnames are case insensitive. e.g. "FooBar" and "foobar" are
        considered equal.

        Arguments
        - obj: the object to compare against.

        Returns
        - `True` if the objects are the same;
                 `False` otherwise.

        See
        - java.net.InetAddress.equals(java.lang.Object)
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a hashcode for this socket address.

        Returns
        - a hash code value for this socket address.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityDamageByBlockEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Function
from org.bukkit.block import Block
from org.bukkit.block import BlockState
from org.bukkit.damage import DamageSource
from org.bukkit.damage import DamageType
from org.bukkit.entity import Entity
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EntityDamageByBlockEvent(EntityDamageEvent):
    """
    Called when an entity is damaged by a block
    """

    def __init__(self, damager: "Block", damagee: "Entity", cause: "DamageCause", damage: float):
        ...


    def __init__(self, damager: "Block", damagerState: "BlockState", damagee: "Entity", cause: "DamageCause", damageSource: "DamageSource", damage: float):
        ...


    def __init__(self, damager: "Block", damagee: "Entity", cause: "DamageCause", modifiers: dict["DamageModifier", "Double"], modifierFunctions: dict["DamageModifier", "Function"["Double", "Double"]]):
        ...


    def __init__(self, damager: "Block", damagerState: "BlockState", damagee: "Entity", cause: "DamageCause", damageSource: "DamageSource", modifiers: dict["DamageModifier", "Double"], modifierFunctions: dict["DamageModifier", "Function"["Double", "Double"]]):
        ...


    def getDamager(self) -> "Block":
        """
        Returns the block that damaged the player.

        Returns
        - Block that damaged the player
        """
        ...


    def getDamagerBlockState(self) -> "BlockState":
        """
        Returns the captured BlockState of the block that damaged the player.

        Returns
        - the block state
        """
        ...



"""
Python module generated from Java source file org.joml.Matrix3x2fStack

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import ObjectInput
from java.io import ObjectOutput
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Matrix3x2fStack(Matrix3x2f):
    """
    A stack of many Matrix3x2f instances. This resembles the matrix stack known from legacy OpenGL.
    
    This Matrix3x2fStack class inherits from Matrix3x2f, so the current/top matrix is always the
    Matrix3x2fStack/Matrix3x2f itself. This affects all operations in Matrix3x2f that take
    another Matrix3x2f as parameter. If a Matrix3x2fStack is used as argument to those methods, the
    effective argument will always be the *current* matrix of the matrix stack.

    Author(s)
    - Kai Burjack
    """

    def __init__(self, stackSize: int):
        """
        Create a new Matrix3x2fStack of the given size.
        
        Initially the stack pointer is at zero and the current matrix is set to identity.

        Arguments
        - stackSize: the size of the stack. This must be at least 1, in which case the Matrix3x2fStack simply only consists of `this`
                   Matrix3x2f
        """
        ...


    def __init__(self):
        """
        Do not invoke manually! Only meant for serialization.
        
        Invoking this constructor from client code will result in an inconsistent state of the 
        created Matrix3x2fStack instance.
        """
        ...


    def clear(self) -> "Matrix3x2fStack":
        """
        Set the stack pointer to zero and set the current/bottom matrix to .identity() identity.

        Returns
        - this
        """
        ...


    def pushMatrix(self) -> "Matrix3x2fStack":
        """
        Increment the stack pointer by one and set the values of the new current matrix to the one directly below it.

        Returns
        - this
        """
        ...


    def popMatrix(self) -> "Matrix3x2fStack":
        """
        Decrement the stack pointer by one.
        
        This will effectively dispose of the current matrix.

        Returns
        - this
        """
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def writeExternal(self, out: "ObjectOutput") -> None:
        ...


    def readExternal(self, in: "ObjectInput") -> None:
        ...


    def clone(self) -> "Object":
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.PotionSplashEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.block import Block
from org.bukkit.block import BlockFace
from org.bukkit.entity import Entity
from org.bukkit.entity import LivingEntity
from org.bukkit.entity import ThrownPotion
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class PotionSplashEvent(ProjectileHitEvent, Cancellable):
    """
    Called when a splash potion hits an area
    """

    def __init__(self, potion: "ThrownPotion", affectedEntities: dict["LivingEntity", "Double"]):
        ...


    def __init__(self, potion: "ThrownPotion", hitEntity: "Entity", hitBlock: "Block", hitFace: "BlockFace", affectedEntities: dict["LivingEntity", "Double"]):
        ...


    def getEntity(self) -> "ThrownPotion":
        ...


    def getPotion(self) -> "ThrownPotion":
        """
        Gets the potion which caused this event

        Returns
        - The thrown potion entity
        """
        ...


    def getAffectedEntities(self) -> Iterable["LivingEntity"]:
        """
        Retrieves a list of all effected entities

        Returns
        - A fresh copy of the affected entity list
        """
        ...


    def getIntensity(self, entity: "LivingEntity") -> float:
        """
        Gets the intensity of the potion's effects for given entity; This
        depends on the distance to the impact center

        Arguments
        - entity: Which entity to get intensity for

        Returns
        - intensity relative to maximum effect; 0.0: not affected; 1.0:
            fully hit by potion effects
        """
        ...


    def setIntensity(self, entity: "LivingEntity", intensity: float) -> None:
        """
        Overwrites the intensity for a given entity

        Arguments
        - entity: For which entity to define a new intensity
        - intensity: relative to maximum effect
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.cache.ForwardingLoadingCache

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import Preconditions
from com.google.common.cache import *
from com.google.common.collect import ImmutableMap
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util.concurrent import ExecutionException
from typing import Any, Callable, Iterable, Tuple


class ForwardingLoadingCache(ForwardingCache, LoadingCache):
    """
    A cache which forwards all its method calls to another cache. Subclasses should override one or
    more methods to modify the behavior of the backing cache as desired per the <a
    href="http://en.wikipedia.org/wiki/Decorator_pattern">decorator pattern</a>.
    
    Note that .get, .getUnchecked, and .apply all expose the same
    underlying functionality, so should probably be overridden as a group.

    Author(s)
    - Charles Fry

    Since
    - 11.0
    """

    def get(self, key: "K") -> "V":
        ...


    def getUnchecked(self, key: "K") -> "V":
        ...


    def getAll(self, keys: Iterable["K"]) -> "ImmutableMap"["K", "V"]:
        ...


    def apply(self, key: "K") -> "V":
        ...


    def refresh(self, key: "K") -> None:
        ...


    class SimpleForwardingLoadingCache(ForwardingLoadingCache):
        """
        A simplified version of ForwardingLoadingCache where subclasses can pass in an already
        constructed LoadingCache as the delegate.

        Since
        - 10.0
        """





"""
Python module generated from Java source file com.google.common.util.concurrent.AsyncCallable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.util.concurrent import *
from java.util.concurrent import Future
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AsyncCallable:
    """
    Computes a value, possibly asynchronously. For an example usage and more information, see Futures.FutureCombiner.callAsync(AsyncCallable, java.util.concurrent.Executor).
    
    Much like java.util.concurrent.Callable, but returning a ListenableFuture
    result.

    Since
    - 20.0
    """

    def call(self) -> "ListenableFuture"["V"]:
        """
        Computes a result `Future`. The output `Future` need not be Future.isDone done, making `AsyncCallable` suitable for asynchronous derivations.
        
        Throwing an exception from this method is equivalent to returning a failing ListenableFuture.
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.MerchantInventory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class MerchantInventory(Inventory):
    """
    Represents a trading inventory between a player and a merchant.
    
    The holder of this Inventory is the owning Villager, or null if the player is
    trading with a merchant created by a plugin.
    """

    def getSelectedRecipeIndex(self) -> int:
        """
        Get the index of the currently selected recipe.

        Returns
        - the index of the currently selected recipe
        """
        ...


    def getSelectedRecipe(self) -> "MerchantRecipe":
        """
        Get the currently active recipe.
        
        This will be `null` if the items provided by the player do
        not match the ingredients of the selected recipe. This does not
        necessarily match the recipe selected by the player: If the player has
        selected the first recipe, the merchant will search all of its offers
        for a matching recipe to activate.

        Returns
        - the currently active recipe
        """
        ...


    def getMerchant(self) -> "Merchant":
        """
        Gets the Merchant associated with this inventory.

        Returns
        - merchant
        """
        ...



"""
Python module generated from Java source file org.bukkit.configuration.InvalidConfigurationException

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.configuration import *
from typing import Any, Callable, Iterable, Tuple


class InvalidConfigurationException(Exception):
    """
    Exception thrown when attempting to load an invalid Configuration
    """

    def __init__(self):
        """
        Creates a new instance of InvalidConfigurationException without a
        message or cause.
        """
        ...


    def __init__(self, msg: str):
        """
        Constructs an instance of InvalidConfigurationException with the
        specified message.

        Arguments
        - msg: The details of the exception.
        """
        ...


    def __init__(self, cause: "Throwable"):
        """
        Constructs an instance of InvalidConfigurationException with the
        specified cause.

        Arguments
        - cause: The cause of the exception.
        """
        ...


    def __init__(self, msg: str, cause: "Throwable"):
        """
        Constructs an instance of InvalidConfigurationException with the
        specified message and cause.

        Arguments
        - cause: The cause of the exception.
        - msg: The details of the exception.
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.Jukebox

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.block import *
from org.bukkit.inventory import BlockInventoryHolder
from org.bukkit.inventory import ItemStack
from org.bukkit.inventory import JukeboxInventory
from typing import Any, Callable, Iterable, Tuple


class Jukebox(TileState, BlockInventoryHolder):
    """
    Represents a captured state of a jukebox.
    """

    def getPlaying(self) -> "Material":
        """
        Gets the record inserted into the jukebox.

        Returns
        - The record Material, or AIR if none is inserted
        """
        ...


    def setPlaying(self, record: "Material") -> None:
        """
        Sets the record being played.

        Arguments
        - record: The record Material, or null/AIR to stop playing
        """
        ...


    def hasRecord(self) -> bool:
        """
        Gets whether or not this jukebox has a record.
        
        A jukebox can have a record but not .isPlaying() be playing
        if it was stopped with .stopPlaying() or if a record has
        finished playing.

        Returns
        - True if this jukebox has a record, False if it the jukebox
        is empty
        """
        ...


    def getRecord(self) -> "ItemStack":
        """
        Gets the record item inserted into the jukebox.

        Returns
        - a copy of the inserted record, or an air stack if none
        """
        ...


    def setRecord(self, record: "ItemStack") -> None:
        """
        Sets the record being played. The jukebox will start playing automatically.

        Arguments
        - record: the record to insert or null/AIR to empty
        """
        ...


    def isPlaying(self) -> bool:
        """
        Checks if the jukebox is playing a record.

        Returns
        - True if there is a record playing
        """
        ...


    def startPlaying(self) -> bool:
        """
        Starts the jukebox playing if there is a record.

        Returns
        - True if the jukebox had a record and was able to start playing, False
        if the jukebox was already playing or did not have a record
        """
        ...


    def stopPlaying(self) -> None:
        """
        Stops the jukebox playing without ejecting the record.
        """
        ...


    def eject(self) -> bool:
        """
        Stops the jukebox playing and ejects the current record.
        
        If the block represented by this state is no longer a jukebox, this will
        do nothing and return False.

        Returns
        - True if a record was ejected; False if there was none playing

        Raises
        - IllegalStateException: if this block state is not placed
        """
        ...


    def getInventory(self) -> "JukeboxInventory":
        """
        Returns
        - inventory

        See
        - Container.getInventory()
        """
        ...


    def getSnapshotInventory(self) -> "JukeboxInventory":
        """
        Returns
        - snapshot inventory

        See
        - Container.getSnapshotInventory()
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.events.SequenceStartEvent

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml.error import Mark
from org.yaml.snakeyaml.events import *
from typing import Any, Callable, Iterable, Tuple


class SequenceStartEvent(CollectionStartEvent):
    """
    Marks the beginning of a sequence node.
    
    This event is followed by the elements contained in the sequence, and a SequenceEndEvent.

    See
    - SequenceEndEvent
    """

    def __init__(self, anchor: str, tag: str, implicit: bool, startMark: "Mark", endMark: "Mark", flowStyle: "DumperOptions.FlowStyle"):
        ...


    def getEventId(self) -> "Event.ID":
        ...



"""
Python module generated from Java source file org.bukkit.event.world.ChunkLoadEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Chunk
from org.bukkit.event import HandlerList
from org.bukkit.event.world import *
from org.bukkit.generator import BlockPopulator
from typing import Any, Callable, Iterable, Tuple


class ChunkLoadEvent(ChunkEvent):
    """
    Called when a chunk is loaded
    """

    def __init__(self, chunk: "Chunk", newChunk: bool):
        ...


    def isNewChunk(self) -> bool:
        """
        Gets if this chunk was newly created or not.
        
        **Note:** Do not use this to generated blocks in a newly generated chunk.
        Use a BlockPopulator instead.

        Returns
        - True if the chunk is new, otherwise False
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.collect.AbstractMultiset

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations.concurrent import LazyInit
from com.google.j2objc.annotations import WeakOuter
from java.util import AbstractCollection
from java.util import Iterator
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AbstractMultiset(AbstractCollection, Multiset):
    """
    This class provides a skeletal implementation of the Multiset interface. A new multiset
    implementation can be created easily by extending this class and implementing the Multiset.entrySet() method, plus optionally overriding .add(Object, int) and .remove(Object, int) to enable modifications to the multiset.
    
    The .count and .size implementations all iterate across the set returned by
    Multiset.entrySet(), as do many methods acting on the set returned by .elementSet(). Override those methods for better performance.

    Author(s)
    - Louis Wasserman
    """

    def isEmpty(self) -> bool:
        ...


    def contains(self, element: "Object") -> bool:
        ...


    def add(self, element: "E") -> bool:
        ...


    def add(self, element: "E", occurrences: int) -> int:
        ...


    def remove(self, element: "Object") -> bool:
        ...


    def remove(self, element: "Object", occurrences: int) -> int:
        ...


    def setCount(self, element: "E", count: int) -> int:
        ...


    def setCount(self, element: "E", oldCount: int, newCount: int) -> bool:
        ...


    def addAll(self, elementsToAdd: Iterable["E"]) -> bool:
        """
        
        
        This implementation is highly efficient when `elementsToAdd` is itself a Multiset.
        """
        ...


    def removeAll(self, elementsToRemove: Iterable[Any]) -> bool:
        ...


    def retainAll(self, elementsToRetain: Iterable[Any]) -> bool:
        ...


    def clear(self) -> None:
        ...


    def elementSet(self) -> set["E"]:
        ...


    def entrySet(self) -> set["Entry"["E"]]:
        ...


    def equals(self, object: "Object") -> bool:
        """
        
        
        This implementation returns `True` if `object` is a multiset of the same size
        and if, for each element, the two multisets have the same count.
        """
        ...


    def hashCode(self) -> int:
        """
        
        
        This implementation returns the hash code of Multiset.entrySet().
        """
        ...


    def toString(self) -> str:
        """
        
        
        This implementation returns the result of invoking `toString` on Multiset.entrySet().
        """
        ...



"""
Python module generated from Java source file org.bukkit.metadata.MetadataStoreBase

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.util import Collections
from java.util import WeakHashMap
from org.bukkit.metadata import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class MetadataStoreBase:

    def setMetadata(self, subject: "T", metadataKey: str, newMetadataValue: "MetadataValue") -> None:
        """
        Adds a metadata value to an object. Each metadata value is owned by a
        specific Plugin. If a plugin has already added a metadata value
        to an object, that value will be replaced with the value of `newMetadataValue`. Multiple plugins can set independent values for the
        same `metadataKey` without conflict.
        
        Implementation note: I considered using a java.util.concurrent.locks.ReadWriteLock for controlling access to
        `metadataMap`, but decided that the added overhead wasn't worth
        the finer grained access control.
        
        Bukkit is almost entirely single threaded so locking overhead shouldn't
        pose a problem.

        Arguments
        - subject: The object receiving the metadata.
        - metadataKey: A unique key to identify this metadata.
        - newMetadataValue: The metadata value to apply.

        Raises
        - IllegalArgumentException: If value is null, or the owning plugin
            is null

        See
        - MetadataStore.setMetadata(Object, String, MetadataValue)
        """
        ...


    def getMetadata(self, subject: "T", metadataKey: str) -> list["MetadataValue"]:
        """
        Returns all metadata values attached to an object. If multiple
        have attached metadata, each will value will be included.

        Arguments
        - subject: the object being interrogated.
        - metadataKey: the unique metadata key being sought.

        Returns
        - A list of values, one for each plugin that has set the
            requested value.

        See
        - MetadataStore.getMetadata(Object, String)
        """
        ...


    def hasMetadata(self, subject: "T", metadataKey: str) -> bool:
        """
        Tests to see if a metadata attribute has been set on an object.

        Arguments
        - subject: the object upon which the has-metadata test is
            performed.
        - metadataKey: the unique metadata key being queried.

        Returns
        - the existence of the metadataKey within subject.
        """
        ...


    def removeMetadata(self, subject: "T", metadataKey: str, owningPlugin: "Plugin") -> None:
        """
        Removes a metadata item owned by a plugin from a subject.

        Arguments
        - subject: the object to remove the metadata from.
        - metadataKey: the unique metadata key identifying the metadata to
            remove.
        - owningPlugin: the plugin attempting to remove a metadata item.

        Raises
        - IllegalArgumentException: If plugin is null

        See
        - MetadataStore.removeMetadata(Object, String,
            org.bukkit.plugin.Plugin)
        """
        ...


    def invalidateAll(self, owningPlugin: "Plugin") -> None:
        """
        Invalidates all metadata in the metadata store that originates from the
        given plugin. Doing this will force each invalidated metadata item to
        be recalculated the next time it is accessed.

        Arguments
        - owningPlugin: the plugin requesting the invalidation.

        Raises
        - IllegalArgumentException: If plugin is null

        See
        - MetadataStore.invalidateAll(org.bukkit.plugin.Plugin)
        """
        ...



"""
Python module generated from Java source file java.nio.file.attribute.AclEntryPermission

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from java.nio.file.attribute import *
from typing import Any, Callable, Iterable, Tuple


class AclEntryPermission(Enum):

# Static fields
    LIST_DIRECTORY = READ_DATA
    """
    Permission to list the entries of a directory (equal to .READ_DATA)
    """
    ADD_FILE = WRITE_DATA
    """
    Permission to add a new file to a directory (equal to .WRITE_DATA)
    """
    ADD_SUBDIRECTORY = APPEND_DATA
    """
    Permission to create a subdirectory to a directory (equal to .APPEND_DATA)
    """


    READ_DATA = 0
    """
    Permission to read the data of the file.
    """
    WRITE_DATA = 1
    """
    Permission to modify the file's data.
    """
    APPEND_DATA = 2
    """
    Permission to append data to a file.
    """
    READ_NAMED_ATTRS = 3
    """
    Permission to read the named attributes of a file.
    
     <a href="http://www.ietf.org/rfc/rfc3530.txt">RFC&nbsp;3530: Network
    File System (NFS) version 4 Protocol</a> defines *named attributes*
    as opaque files associated with a file in the file system.
    """
    WRITE_NAMED_ATTRS = 4
    """
    Permission to write the named attributes of a file.
    
     <a href="http://www.ietf.org/rfc/rfc3530.txt">RFC&nbsp;3530: Network
    File System (NFS) version 4 Protocol</a> defines *named attributes*
    as opaque files associated with a file in the file system.
    """
    EXECUTE = 5
    """
    Permission to execute a file.
    """
    DELETE_CHILD = 6
    """
    Permission to delete a file or directory within a directory.
    """
    READ_ATTRIBUTES = 7
    """
    The ability to read (non-acl) file attributes.
    """
    WRITE_ATTRIBUTES = 8
    """
    The ability to write (non-acl) file attributes.
    """
    DELETE = 9
    """
    Permission to delete the file.
    """
    READ_ACL = 10
    """
    Permission to read the ACL attribute.
    """
    WRITE_ACL = 11
    """
    Permission to write the ACL attribute.
    """
    WRITE_OWNER = 12
    """
    Permission to change the owner.
    """
    SYNCHRONIZE = 13
    """
    Permission to access file locally at the server with synchronous reads
    and writes.
    """



"""
Python module generated from Java source file java.util.concurrent.Semaphore

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent import *
from java.util.concurrent.locks import AbstractQueuedSynchronizer
from typing import Any, Callable, Iterable, Tuple


class Semaphore(Serializable):
    """
    A counting semaphore.  Conceptually, a semaphore maintains a set of
    permits.  Each .acquire blocks if necessary until a permit is
    available, and then takes it.  Each .release adds a permit,
    potentially releasing a blocking acquirer.
    However, no actual permit objects are used; the `Semaphore` just
    keeps a count of the number available and acts accordingly.
    
    Semaphores are often used to restrict the number of threads than can
    access some (physical or logical) resource. For example, here is
    a class that uses a semaphore to control access to a pool of items:
    ``` `class Pool {
      private static final int MAX_AVAILABLE = 100;
      private final Semaphore available = new Semaphore(MAX_AVAILABLE, True);
    
      public Object getItem() throws InterruptedException {
        available.acquire();
        return getNextAvailableItem();`
    
      public void putItem(Object x) {
        if (markAsUnused(x))
          available.release();
      }
    
      // Not a particularly efficient data structure; just for demo
    
      protected Object[] items = ...; // whatever kinds of items being managed
      protected boolean[] used = new boolean[MAX_AVAILABLE];
    
      protected synchronized Object getNextAvailableItem() {
        for (int i = 0; i < MAX_AVAILABLE; ++i) {
          if (!used[i]) {
            used[i] = True;
            return items[i];
          }
        }
        return null; // not reached
      }
    
      protected synchronized boolean markAsUnused(Object item) {
        for (int i = 0; i < MAX_AVAILABLE; ++i) {
          if (item == items[i]) {
            if (used[i]) {
              used[i] = False;
              return True;
            } else
              return False;
          }
        }
        return False;
      }
    }}```
    
    Before obtaining an item each thread must acquire a permit from
    the semaphore, guaranteeing that an item is available for use. When
    the thread has finished with the item it is returned back to the
    pool and a permit is returned to the semaphore, allowing another
    thread to acquire that item.  Note that no synchronization lock is
    held when .acquire is called as that would prevent an item
    from being returned to the pool.  The semaphore encapsulates the
    synchronization needed to restrict access to the pool, separately
    from any synchronization needed to maintain the consistency of the
    pool itself.
    
    A semaphore initialized to one, and which is used such that it
    only has at most one permit available, can serve as a mutual
    exclusion lock.  This is more commonly known as a *binary
    semaphore*, because it only has two states: one permit
    available, or zero permits available.  When used in this way, the
    binary semaphore has the property (unlike many java.util.concurrent.locks.Lock
    implementations), that the &quot;lock&quot; can be released by a
    thread other than the owner (as semaphores have no notion of
    ownership).  This can be useful in some specialized contexts, such
    as deadlock recovery.
    
    The constructor for this class optionally accepts a
    *fairness* parameter. When set False, this class makes no
    guarantees about the order in which threads acquire permits. In
    particular, *barging* is permitted, that is, a thread
    invoking .acquire can be allocated a permit ahead of a
    thread that has been waiting - logically the new thread places itself at
    the head of the queue of waiting threads. When fairness is set True, the
    semaphore guarantees that threads invoking any of the .acquire() acquire methods are selected to obtain permits in the order in
    which their invocation of those methods was processed
    (first-in-first-out; FIFO). Note that FIFO ordering necessarily
    applies to specific internal points of execution within these
    methods.  So, it is possible for one thread to invoke
    `acquire` before another, but reach the ordering point after
    the other, and similarly upon return from the method.
    Also note that the untimed .tryAcquire() tryAcquire methods do not
    honor the fairness setting, but will take any permits that are
    available.
    
    Generally, semaphores used to control resource access should be
    initialized as fair, to ensure that no thread is starved out from
    accessing a resource. When using semaphores for other kinds of
    synchronization control, the throughput advantages of non-fair
    ordering often outweigh fairness considerations.
    
    This class also provides convenience methods to .acquire(int) acquire and .release(int) release multiple
    permits at a time. These methods are generally more efficient and
    effective than loops. However, they do not establish any preference
    order. For example, if thread A invokes `s.acquire(3`) and
    thread B invokes `s.acquire(2)`, and two permits become
    available, then there is no guarantee that thread B will obtain
    them unless its acquire came first and Semaphore `s` is in
    fair mode.
    
    Memory consistency effects: Actions in a thread prior to calling
    a "release" method such as `release()`
    <a href="package-summary.html#MemoryVisibility">*happen-before*</a>
    actions following a successful "acquire" method such as `acquire()`
    in another thread.

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def __init__(self, permits: int):
        """
        Creates a `Semaphore` with the given number of
        permits and nonfair fairness setting.

        Arguments
        - permits: the initial number of permits available.
               This value may be negative, in which case releases
               must occur before any acquires will be granted.
        """
        ...


    def __init__(self, permits: int, fair: bool):
        """
        Creates a `Semaphore` with the given number of
        permits and the given fairness setting.

        Arguments
        - permits: the initial number of permits available.
               This value may be negative, in which case releases
               must occur before any acquires will be granted.
        - fair: `True` if this semaphore will guarantee
               first-in first-out granting of permits under contention,
               else `False`
        """
        ...


    def acquire(self) -> None:
        """
        Acquires a permit from this semaphore, blocking until one is
        available, or the thread is Thread.interrupt interrupted.
        
        Acquires a permit, if one is available and returns immediately,
        reducing the number of available permits by one.
        
        If no permit is available then the current thread becomes
        disabled for thread scheduling purposes and lies dormant until
        one of two things happens:
        
        - Some other thread invokes the .release method for this
        semaphore and the current thread is next to be assigned a permit; or
        - Some other thread Thread.interrupt interrupts
        the current thread.
        
        
        If the current thread:
        
        - has its interrupted status set on entry to this method; or
        - is Thread.interrupt interrupted while waiting
        for a permit,
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared.

        Raises
        - InterruptedException: if the current thread is interrupted
        """
        ...


    def acquireUninterruptibly(self) -> None:
        """
        Acquires a permit from this semaphore, blocking until one is
        available.
        
        Acquires a permit, if one is available and returns immediately,
        reducing the number of available permits by one.
        
        If no permit is available then the current thread becomes
        disabled for thread scheduling purposes and lies dormant until
        some other thread invokes the .release method for this
        semaphore and the current thread is next to be assigned a permit.
        
        If the current thread is Thread.interrupt interrupted
        while waiting for a permit then it will continue to wait, but the
        time at which the thread is assigned a permit may change compared to
        the time it would have received the permit had no interruption
        occurred.  When the thread does return from this method its interrupt
        status will be set.
        """
        ...


    def tryAcquire(self) -> bool:
        """
        Acquires a permit from this semaphore, only if one is available at the
        time of invocation.
        
        Acquires a permit, if one is available and returns immediately,
        with the value `True`,
        reducing the number of available permits by one.
        
        If no permit is available then this method will return
        immediately with the value `False`.
        
        Even when this semaphore has been set to use a
        fair ordering policy, a call to `tryAcquire()` *will*
        immediately acquire a permit if one is available, whether or not
        other threads are currently waiting.
        This &quot;barging&quot; behavior can be useful in certain
        circumstances, even though it breaks fairness. If you want to honor
        the fairness setting, then use
        .tryAcquire(long, TimeUnit) tryAcquire(0, TimeUnit.SECONDS)
        which is almost equivalent (it also detects interruption).

        Returns
        - `True` if a permit was acquired and `False`
                otherwise
        """
        ...


    def tryAcquire(self, timeout: int, unit: "TimeUnit") -> bool:
        """
        Acquires a permit from this semaphore, if one becomes available
        within the given waiting time and the current thread has not
        been Thread.interrupt interrupted.
        
        Acquires a permit, if one is available and returns immediately,
        with the value `True`,
        reducing the number of available permits by one.
        
        If no permit is available then the current thread becomes
        disabled for thread scheduling purposes and lies dormant until
        one of three things happens:
        
        - Some other thread invokes the .release method for this
        semaphore and the current thread is next to be assigned a permit; or
        - Some other thread Thread.interrupt interrupts
        the current thread; or
        - The specified waiting time elapses.
        
        
        If a permit is acquired then the value `True` is returned.
        
        If the current thread:
        
        - has its interrupted status set on entry to this method; or
        - is Thread.interrupt interrupted while waiting
        to acquire a permit,
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared.
        
        If the specified waiting time elapses then the value `False`
        is returned.  If the time is less than or equal to zero, the method
        will not wait at all.

        Arguments
        - timeout: the maximum time to wait for a permit
        - unit: the time unit of the `timeout` argument

        Returns
        - `True` if a permit was acquired and `False`
                if the waiting time elapsed before a permit was acquired

        Raises
        - InterruptedException: if the current thread is interrupted
        """
        ...


    def release(self) -> None:
        """
        Releases a permit, returning it to the semaphore.
        
        Releases a permit, increasing the number of available permits by
        one.  If any threads are trying to acquire a permit, then one is
        selected and given the permit that was just released.  That thread
        is (re)enabled for thread scheduling purposes.
        
        There is no requirement that a thread that releases a permit must
        have acquired that permit by calling .acquire.
        Correct usage of a semaphore is established by programming convention
        in the application.
        """
        ...


    def acquire(self, permits: int) -> None:
        """
        Acquires the given number of permits from this semaphore,
        blocking until all are available,
        or the thread is Thread.interrupt interrupted.
        
        Acquires the given number of permits, if they are available,
        and returns immediately, reducing the number of available permits
        by the given amount. This method has the same effect as the
        loop `for (int i = 0; i < permits; ++i) acquire();` except
        that it atomically acquires the permits all at once:
        
        If insufficient permits are available then the current thread becomes
        disabled for thread scheduling purposes and lies dormant until
        one of two things happens:
        
        - Some other thread invokes one of the .release() release
        methods for this semaphore and the current thread is next to be assigned
        permits and the number of available permits satisfies this request; or
        - Some other thread Thread.interrupt interrupts
        the current thread.
        
        
        If the current thread:
        
        - has its interrupted status set on entry to this method; or
        - is Thread.interrupt interrupted while waiting
        for a permit,
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared.
        Any permits that were to be assigned to this thread are instead
        assigned to other threads trying to acquire permits, as if
        permits had been made available by a call to .release().

        Arguments
        - permits: the number of permits to acquire

        Raises
        - InterruptedException: if the current thread is interrupted
        - IllegalArgumentException: if `permits` is negative
        """
        ...


    def acquireUninterruptibly(self, permits: int) -> None:
        """
        Acquires the given number of permits from this semaphore,
        blocking until all are available.
        
        Acquires the given number of permits, if they are available,
        and returns immediately, reducing the number of available permits
        by the given amount. This method has the same effect as the
        loop `for (int i = 0; i < permits; ++i) acquireUninterruptibly();`
        except that it atomically acquires the permits all at once:
        
        If insufficient permits are available then the current thread becomes
        disabled for thread scheduling purposes and lies dormant until
        some other thread invokes one of the .release() release
        methods for this semaphore and the current thread is next to be assigned
        permits and the number of available permits satisfies this request.
        
        If the current thread is Thread.interrupt interrupted
        while waiting for permits then it will continue to wait and its
        position in the queue is not affected.  When the thread does return
        from this method its interrupt status will be set.

        Arguments
        - permits: the number of permits to acquire

        Raises
        - IllegalArgumentException: if `permits` is negative
        """
        ...


    def tryAcquire(self, permits: int) -> bool:
        """
        Acquires the given number of permits from this semaphore, only
        if all are available at the time of invocation.
        
        Acquires the given number of permits, if they are available, and
        returns immediately, with the value `True`,
        reducing the number of available permits by the given amount.
        
        If insufficient permits are available then this method will return
        immediately with the value `False` and the number of available
        permits is unchanged.
        
        Even when this semaphore has been set to use a fair ordering
        policy, a call to `tryAcquire` *will*
        immediately acquire a permit if one is available, whether or
        not other threads are currently waiting.  This
        &quot;barging&quot; behavior can be useful in certain
        circumstances, even though it breaks fairness. If you want to
        honor the fairness setting, then use .tryAcquire(int,
        long, TimeUnit) tryAcquire(permits, 0, TimeUnit.SECONDS)
        which is almost equivalent (it also detects interruption).

        Arguments
        - permits: the number of permits to acquire

        Returns
        - `True` if the permits were acquired and
                `False` otherwise

        Raises
        - IllegalArgumentException: if `permits` is negative
        """
        ...


    def tryAcquire(self, permits: int, timeout: int, unit: "TimeUnit") -> bool:
        """
        Acquires the given number of permits from this semaphore, if all
        become available within the given waiting time and the current
        thread has not been Thread.interrupt interrupted.
        
        Acquires the given number of permits, if they are available and
        returns immediately, with the value `True`,
        reducing the number of available permits by the given amount.
        
        If insufficient permits are available then
        the current thread becomes disabled for thread scheduling
        purposes and lies dormant until one of three things happens:
        
        - Some other thread invokes one of the .release() release
        methods for this semaphore and the current thread is next to be assigned
        permits and the number of available permits satisfies this request; or
        - Some other thread Thread.interrupt interrupts
        the current thread; or
        - The specified waiting time elapses.
        
        
        If the permits are acquired then the value `True` is returned.
        
        If the current thread:
        
        - has its interrupted status set on entry to this method; or
        - is Thread.interrupt interrupted while waiting
        to acquire the permits,
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared.
        Any permits that were to be assigned to this thread, are instead
        assigned to other threads trying to acquire permits, as if
        the permits had been made available by a call to .release().
        
        If the specified waiting time elapses then the value `False`
        is returned.  If the time is less than or equal to zero, the method
        will not wait at all.  Any permits that were to be assigned to this
        thread, are instead assigned to other threads trying to acquire
        permits, as if the permits had been made available by a call to
        .release().

        Arguments
        - permits: the number of permits to acquire
        - timeout: the maximum time to wait for the permits
        - unit: the time unit of the `timeout` argument

        Returns
        - `True` if all permits were acquired and `False`
                if the waiting time elapsed before all permits were acquired

        Raises
        - InterruptedException: if the current thread is interrupted
        - IllegalArgumentException: if `permits` is negative
        """
        ...


    def release(self, permits: int) -> None:
        """
        Releases the given number of permits, returning them to the semaphore.
        
        Releases the given number of permits, increasing the number of
        available permits by that amount.
        If any threads are trying to acquire permits, then one thread
        is selected and given the permits that were just released.
        If the number of available permits satisfies that thread's request
        then that thread is (re)enabled for thread scheduling purposes;
        otherwise the thread will wait until sufficient permits are available.
        If there are still permits available
        after this thread's request has been satisfied, then those permits
        are assigned in turn to other threads trying to acquire permits.
        
        There is no requirement that a thread that releases a permit must
        have acquired that permit by calling Semaphore.acquire acquire.
        Correct usage of a semaphore is established by programming convention
        in the application.

        Arguments
        - permits: the number of permits to release

        Raises
        - IllegalArgumentException: if `permits` is negative
        """
        ...


    def availablePermits(self) -> int:
        """
        Returns the current number of permits available in this semaphore.
        
        This method is typically used for debugging and testing purposes.

        Returns
        - the number of permits available in this semaphore
        """
        ...


    def drainPermits(self) -> int:
        """
        Acquires and returns all permits that are immediately
        available, or if negative permits are available, releases them.
        Upon return, zero permits are available.

        Returns
        - the number of permits acquired or, if negative, the
        number released
        """
        ...


    def isFair(self) -> bool:
        """
        Returns `True` if this semaphore has fairness set True.

        Returns
        - `True` if this semaphore has fairness set True
        """
        ...


    def hasQueuedThreads(self) -> bool:
        """
        Queries whether any threads are waiting to acquire. Note that
        because cancellations may occur at any time, a `True`
        return does not guarantee that any other thread will ever
        acquire.  This method is designed primarily for use in
        monitoring of the system state.

        Returns
        - `True` if there may be other threads waiting to
                acquire the lock
        """
        ...


    def getQueueLength(self) -> int:
        """
        Returns an estimate of the number of threads waiting to acquire.
        The value is only an estimate because the number of threads may
        change dynamically while this method traverses internal data
        structures.  This method is designed for use in monitoring
        system state, not for synchronization control.

        Returns
        - the estimated number of threads waiting for this lock
        """
        ...


    def toString(self) -> str:
        """
        Returns a string identifying this semaphore, as well as its state.
        The state, in brackets, includes the String `"Permits ="`
        followed by the number of permits.

        Returns
        - a string identifying this semaphore, as well as its state
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockDispenseLootEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class BlockDispenseLootEvent(BlockEvent, Cancellable):
    """
    Called when a block dispenses loot from its designated LootTable. This is not
    to be confused with events like BlockDispenseEvent which fires when a
    singular item is dispensed from its inventory container.
    
    Example: A player unlocks a trial chamber vault and the vault block dispenses
    its loot.
    """

    def __init__(self, player: "Player", theBlock: "Block", dispensedLoot: list["ItemStack"]):
        ...


    def getDispensedLoot(self) -> list["ItemStack"]:
        """
        Gets the loot that will be dispensed.

        Returns
        - the loot that will be dispensed
        """
        ...


    def setDispensedLoot(self, dispensedLoot: list["ItemStack"]) -> None:
        """
        Sets the loot that will be dispensed.

        Arguments
        - dispensedLoot: new loot to dispense
        """
        ...


    def getPlayer(self) -> "Player":
        """
        Gets the player associated with this event.
        
        **Warning:** Some event instances like a
        org.bukkit.block.TrialSpawner dispensing its reward loot may not
        have a player associated with them and will return null.

        Returns
        - the player who unlocked the vault
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancelled: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.joml.ConfigurationException

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class ConfigurationException(RuntimeException):
    """
    Exception thrown when using an invalid JOML runtime configuration.

    Author(s)
    - Kai Burjack
    """

    def __init__(self, message: str, cause: "Throwable"):
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.AbstractScheduledService

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations.concurrent import GuardedBy
from com.google.j2objc.annotations import WeakOuter
from java.time import Duration
from java.util.concurrent import Callable
from java.util.concurrent import Executor
from java.util.concurrent import Executors
from java.util.concurrent import Future
from java.util.concurrent import ScheduledExecutorService
from java.util.concurrent import ScheduledFuture
from java.util.concurrent import ThreadFactory
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from java.util.concurrent.locks import ReentrantLock
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AbstractScheduledService(Service):
    """
    Base class for services that can implement .startUp and .shutDown but while in
    the "running" state need to perform a periodic task. Subclasses can implement .startUp,
    .shutDown and also a .runOneIteration method that will be executed periodically.
    
    This class uses the ScheduledExecutorService returned from .executor to run
    the .startUp and .shutDown methods and also uses that service to schedule the
    .runOneIteration that will be executed periodically as specified by its Scheduler. When this service is asked to stop via .stopAsync it will cancel the periodic
    task (but not interrupt it) and wait for it to stop before running the .shutDown method.
    
    Subclasses are guaranteed that the life cycle methods (.runOneIteration, .startUp and .shutDown) will never run concurrently. Notably, if any execution of .runOneIteration takes longer than its schedule defines, then subsequent executions may start
    late. Also, all life cycle methods are executed with a lock held, so subclasses can safely modify
    shared state without additional synchronization necessary for visibility to later executions of
    the life cycle methods.
    
    <h3>Usage Example</h3>
    
    Here is a sketch of a service which crawls a website and uses the scheduling capabilities to
    rate limit itself.
    
    ````class CrawlingService extends AbstractScheduledService {
      private Set<Uri> visited;
      private Queue<Uri> toCrawl;
      protected void startUp() throws Exception {
        toCrawl = readStartingUris();`
    
      protected void runOneIteration() throws Exception {
        Uri uri = toCrawl.remove();
        Collection<Uri> newUris = crawl(uri);
        visited.add(uri);
        for (Uri newUri : newUris) {
          if (!visited.contains(newUri)) { toCrawl.add(newUri); }
        }
      }
    
      protected void shutDown() throws Exception {
        saveUris(toCrawl);
      }
    
      protected Scheduler scheduler() {
        return Scheduler.newFixedRateSchedule(0, 1, TimeUnit.SECONDS);
      }
    }
    }```
    
    This class uses the life cycle methods to read in a list of starting URIs and save the set of
    outstanding URIs when shutting down. Also, it takes advantage of the scheduling functionality to
    rate limit the number of queries we perform.

    Author(s)
    - Luke Sandberg

    Since
    - 11.0
    """

    def toString(self) -> str:
        ...


    def isRunning(self) -> bool:
        ...


    def state(self) -> "State":
        ...


    def addListener(self, listener: "Listener", executor: "Executor") -> None:
        """
        Since
        - 13.0
        """
        ...


    def failureCause(self) -> "Throwable":
        """
        Since
        - 14.0
        """
        ...


    def startAsync(self) -> "Service":
        """
        Since
        - 15.0
        """
        ...


    def stopAsync(self) -> "Service":
        """
        Since
        - 15.0
        """
        ...


    def awaitRunning(self) -> None:
        """
        Since
        - 15.0
        """
        ...


    def awaitRunning(self, timeout: "Duration") -> None:
        """
        Since
        - 28.0
        """
        ...


    def awaitRunning(self, timeout: int, unit: "TimeUnit") -> None:
        """
        Since
        - 15.0
        """
        ...


    def awaitTerminated(self) -> None:
        """
        Since
        - 15.0
        """
        ...


    def awaitTerminated(self, timeout: "Duration") -> None:
        """
        Since
        - 28.0
        """
        ...


    def awaitTerminated(self, timeout: int, unit: "TimeUnit") -> None:
        """
        Since
        - 15.0
        """
        ...


    class Scheduler:
        """
        A scheduler defines the policy for how the AbstractScheduledService should run its
        task.
        
        Consider using the .newFixedDelaySchedule and .newFixedRateSchedule factory
        methods, these provide Scheduler instances for the common use case of running the
        service with a fixed schedule. If more flexibility is needed then consider subclassing CustomScheduler.

    Author(s)
        - Luke Sandberg

        Since
        - 11.0
        """

        @staticmethod
        def newFixedDelaySchedule(initialDelay: "Duration", delay: "Duration") -> "Scheduler":
            """
            Returns a Scheduler that schedules the task using the ScheduledExecutorService.scheduleWithFixedDelay method.

            Arguments
            - initialDelay: the time to delay first execution
            - delay: the delay between the termination of one execution and the commencement of the
                next

            Since
            - 28.0
            """
            ...


        @staticmethod
        def newFixedDelaySchedule(initialDelay: int, delay: int, unit: "TimeUnit") -> "Scheduler":
            """
            Returns a Scheduler that schedules the task using the ScheduledExecutorService.scheduleWithFixedDelay method.

            Arguments
            - initialDelay: the time to delay first execution
            - delay: the delay between the termination of one execution and the commencement of the
                next
            - unit: the time unit of the initialDelay and delay parameters
            """
            ...


        @staticmethod
        def newFixedRateSchedule(initialDelay: "Duration", period: "Duration") -> "Scheduler":
            """
            Returns a Scheduler that schedules the task using the ScheduledExecutorService.scheduleAtFixedRate method.

            Arguments
            - initialDelay: the time to delay first execution
            - period: the period between successive executions of the task

            Since
            - 28.0
            """
            ...


        @staticmethod
        def newFixedRateSchedule(initialDelay: int, period: int, unit: "TimeUnit") -> "Scheduler":
            """
            Returns a Scheduler that schedules the task using the ScheduledExecutorService.scheduleAtFixedRate method.

            Arguments
            - initialDelay: the time to delay first execution
            - period: the period between successive executions of the task
            - unit: the time unit of the initialDelay and period parameters
            """
            ...


    class CustomScheduler(Scheduler):
        """
        A Scheduler that provides a convenient way for the AbstractScheduledService to
        use a dynamically changing schedule. After every execution of the task, assuming it hasn't been
        cancelled, the .getNextSchedule method will be called.

    Author(s)
        - Luke Sandberg

        Since
        - 11.0
        """





"""
Python module generated from Java source file org.bukkit.Warning

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import ImmutableMap
from java.util import Locale
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class Warning:
    """
    This designates the warning state for a specific item.
    
    When the server settings dictate 'default' warnings, warnings are printed
    if the .value() is True.
    """

    def value(self) -> bool:
        """
        This sets if the deprecation warnings when registering events gets
        printed when the setting is in the default state.

        Returns
        - False normally, or True to encourage warning printout
        """
        return False


    def reason(self) -> str:
        """
        This can provide detailed information on why the event is deprecated.

        Returns
        - The reason an event is deprecated
        """
        return ""



"""
Python module generated from Java source file org.bukkit.attribute.Attribute

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import Lists
from java.util import Locale
from org.bukkit import Bukkit
from org.bukkit import Keyed
from org.bukkit import NamespacedKey
from org.bukkit import Registry
from org.bukkit import Translatable
from org.bukkit.attribute import *
from org.bukkit.registry import RegistryAware
from org.bukkit.util import OldEnum
from typing import Any, Callable, Iterable, Tuple


class Attribute(OldEnum, Keyed, Translatable, RegistryAware):
    """
    Types of attributes which may be present on an Attributable.
    """

    MAX_HEALTH = getAttribute("max_health")
    """
    Maximum health of an Entity.
    """
    FOLLOW_RANGE = getAttribute("follow_range")
    """
    Range at which an Entity will follow others.
    """
    KNOCKBACK_RESISTANCE = getAttribute("knockback_resistance")
    """
    Resistance of an Entity to knockback.
    """
    MOVEMENT_SPEED = getAttribute("movement_speed")
    """
    Movement speed of an Entity.
    """
    FLYING_SPEED = getAttribute("flying_speed")
    """
    Flying speed of an Entity.
    """
    ATTACK_DAMAGE = getAttribute("attack_damage")
    """
    Attack damage of an Entity.
    """
    ATTACK_KNOCKBACK = getAttribute("attack_knockback")
    """
    Attack knockback of an Entity.
    """
    ATTACK_SPEED = getAttribute("attack_speed")
    """
    Attack speed of an Entity.
    """
    ARMOR = getAttribute("armor")
    """
    Armor bonus of an Entity.
    """
    ARMOR_TOUGHNESS = getAttribute("armor_toughness")
    """
    Armor durability bonus of an Entity.
    """
    FALL_DAMAGE_MULTIPLIER = getAttribute("fall_damage_multiplier")
    """
    The fall damage multiplier of an Entity.
    """
    LUCK = getAttribute("luck")
    """
    Luck bonus of an Entity.
    """
    MAX_ABSORPTION = getAttribute("max_absorption")
    """
    Maximum absorption of an Entity.
    """
    SAFE_FALL_DISTANCE = getAttribute("safe_fall_distance")
    """
    The distance which an Entity can fall without damage.
    """
    SCALE = getAttribute("scale")
    """
    The relative scale of an Entity.
    """
    STEP_HEIGHT = getAttribute("step_height")
    """
    The height which an Entity can walk over.
    """
    GRAVITY = getAttribute("gravity")
    """
    The gravity applied to an Entity.
    """
    JUMP_STRENGTH = getAttribute("jump_strength")
    """
    Strength with which an Entity will jump.
    """
    BURNING_TIME = getAttribute("burning_time")
    """
    How long an entity remains burning after ignition.
    """
    EXPLOSION_KNOCKBACK_RESISTANCE = getAttribute("explosion_knockback_resistance")
    """
    Resistance to knockback from explosions.
    """
    MOVEMENT_EFFICIENCY = getAttribute("movement_efficiency")
    """
    Movement speed through difficult terrain.
    """
    OXYGEN_BONUS = getAttribute("oxygen_bonus")
    """
    Oxygen use underwater.
    """
    WATER_MOVEMENT_EFFICIENCY = getAttribute("water_movement_efficiency")
    """
    Movement speed through water.
    """
    TEMPT_RANGE = getAttribute("tempt_range")
    """
    Range at which mobs will be tempted by items.
    """
    BLOCK_INTERACTION_RANGE = getAttribute("block_interaction_range")
    """
    The block reach distance of a Player.
    """
    ENTITY_INTERACTION_RANGE = getAttribute("entity_interaction_range")
    """
    The entity reach distance of a Player.
    """
    BLOCK_BREAK_SPEED = getAttribute("block_break_speed")
    """
    Block break speed of a Player.
    """
    MINING_EFFICIENCY = getAttribute("mining_efficiency")
    """
    Mining speed for correct tools.
    """
    SNEAKING_SPEED = getAttribute("sneaking_speed")
    """
    Sneaking speed.
    """
    SUBMERGED_MINING_SPEED = getAttribute("submerged_mining_speed")
    """
    Underwater mining speed.
    """
    SWEEPING_DAMAGE_RATIO = getAttribute("sweeping_damage_ratio")
    """
    Sweeping damage.
    """
    SPAWN_REINFORCEMENTS = getAttribute("spawn_reinforcements")
    """
    Chance of a zombie to spawn reinforcements.
    """


    @staticmethod
    def getAttribute(key: str) -> "Attribute":
        ...


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...


    @staticmethod
    def valueOf(name: str) -> "Attribute":
        """
        Arguments
        - name: of the attribute.

        Returns
        - the attribute with the given name.

        Deprecated
        - only for backwards compatibility, use Registry.get(NamespacedKey) instead.
        """
        ...


    @staticmethod
    def values() -> list["Attribute"]:
        """
        Returns
        - an array of all known attributes.

        Deprecated
        - use Registry.iterator().
        """
        ...



"""
Python module generated from Java source file org.bukkit.advancement.AdvancementDisplay

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.advancement import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class AdvancementDisplay:
    """
    Holds information about how the advancement is displayed by the game.
    """

    def getTitle(self) -> str:
        """
        Gets the title of the advancement.

        Returns
        - The advancement title without colour codes.
        """
        ...


    def getDescription(self) -> str:
        """
        Gets the visible description of the advancement.

        Returns
        - The advancement description without colour codes.
        """
        ...


    def getIcon(self) -> "ItemStack":
        """
        The icon that is used for this advancement.

        Returns
        - an ItemStack that represents the advancement.
        """
        ...


    def shouldShowToast(self) -> bool:
        """
        Whether to show a toast to the player when this advancement has been
        completed.

        Returns
        - True if a toast is shown.
        """
        ...


    def shouldAnnounceChat(self) -> bool:
        """
        Whether to announce in the chat when this advancement has been completed.

        Returns
        - True if announced in chat.
        """
        ...


    def isHidden(self) -> bool:
        """
        Whether to hide this advancement and all its children from the
        advancement screen until this advancement have been completed.
        
        Has no effect on root advancements themselves, but still affects all
        their children.

        Returns
        - True if hidden.
        """
        ...


    def getX(self) -> float:
        """
        The X position of the advancement in the advancement screen.

        Returns
        - the X coordinate as float
        """
        ...


    def getY(self) -> float:
        """
        The Y position of the advancement in the advancement screen.

        Returns
        - the Y coordinate as float
        """
        ...


    def getType(self) -> "AdvancementDisplayType":
        """
        The display type of this advancement.

        Returns
        - an enum representing the type.
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.PigZombie

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class PigZombie(Zombie):
    """
    Represents a Pig Zombie.
    """

    def getAnger(self) -> int:
        """
        Get the pig zombie's current anger level.

        Returns
        - The anger level.
        """
        ...


    def setAnger(self, level: int) -> None:
        """
        Set the pig zombie's current anger level.

        Arguments
        - level: The anger level. Higher levels of anger take longer to
            wear off.
        """
        ...


    def setAngry(self, angry: bool) -> None:
        """
        Shorthand; sets to either 0 or the default level.

        Arguments
        - angry: Whether the zombie should be angry.
        """
        ...


    def isAngry(self) -> bool:
        """
        Shorthand; gets whether the zombie is angry.

        Returns
        - True if the zombie is angry, otherwise False.
        """
        ...


    def isConverting(self) -> bool:
        """
        **Not applicable to this entity**

        Returns
        - False
        """
        ...


    def getConversionTime(self) -> int:
        """
        **Not applicable to this entity**

        Returns
        - UnsuppotedOperationException
        """
        ...


    def setConversionTime(self, time: int) -> None:
        """
        **Not applicable to this entity**

        Arguments
        - time: unused
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.SparseImmutableTable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import Immutable
from typing import Any, Callable, Iterable, Tuple


class SparseImmutableTable(RegularImmutableTable):
    """
    A `RegularImmutableTable` optimized for sparse data.
    """

    def columnMap(self) -> "ImmutableMap"["C", dict["R", "V"]]:
        ...


    def rowMap(self) -> "ImmutableMap"["R", dict["C", "V"]]:
        ...


    def size(self) -> int:
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.LoaderOptions

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml import *
from org.yaml.snakeyaml.inspector import TagInspector
from org.yaml.snakeyaml.inspector import UnTrustedTagInspector
from typing import Any, Callable, Iterable, Tuple


class LoaderOptions:
    """
    Configuration for loading
    """

    def isAllowDuplicateKeys(self) -> bool:
        """
        getter

        Returns
        - True when duplicate keys in mapping allowed (the latter overrides the former)
        """
        ...


    def setAllowDuplicateKeys(self, allowDuplicateKeys: bool) -> None:
        """
        Allow/Reject duplicate map keys in the YAML file.
        
        Default is to allow.
        
        YAML 1.1 is slightly vague around duplicate entries in the YAML file. The best reference is
        <a href="http://www.yaml.org/spec/1.1/#id862121"> 3.2.1.3. Nodes Comparison</a> where it hints
        that a duplicate map key is an error.
        
        For future reference, YAML spec 1.2 is clear. The keys MUST be unique.
        <a href="http://www.yaml.org/spec/1.2/spec.html#id2759572">1.3. Relation to JSON</a>

        Arguments
        - allowDuplicateKeys: False to reject duplicate mapping keys
        """
        ...


    def isWrappedToRootException(self) -> bool:
        """
        getter

        Returns
        - True when wrapped
        """
        ...


    def setWrappedToRootException(self, wrappedToRootException: bool) -> None:
        """
        Wrap runtime exception to YAMLException during parsing or leave them as they are
        
        Default is to leave original exceptions

        Arguments
        - wrappedToRootException: - True to convert runtime exception to YAMLException
        """
        ...


    def getMaxAliasesForCollections(self) -> int:
        """
        getter

        Returns
        - show the limit
        """
        ...


    def setMaxAliasesForCollections(self, maxAliasesForCollections: int) -> None:
        """
        Restrict the amount of aliases for collections (sequences and mappings) to avoid
        https://en.wikipedia.org/wiki/Billion_laughs_attack

        Arguments
        - maxAliasesForCollections: set max allowed value (50 by default)
        """
        ...


    def getAllowRecursiveKeys(self) -> bool:
        """
        getter

        Returns
        - when recursive keys are allowed (the document should be trusted)
        """
        ...


    def setAllowRecursiveKeys(self, allowRecursiveKeys: bool) -> None:
        """
        Allow recursive keys for mappings. By default, it is not allowed. This setting only prevents
        the case when the key is the value. If the key is only a part of the value (the value is a
        sequence or a mapping) then this case is not recognized and always allowed.

        Arguments
        - allowRecursiveKeys: - False to disable recursive keys
        """
        ...


    def isProcessComments(self) -> bool:
        """
        getter

        Returns
        - comments are kept in Node
        """
        ...


    def setProcessComments(self, processComments: bool) -> "LoaderOptions":
        """
        Set the comment processing. By default, comments are ignored.

        Arguments
        - processComments: `True` to process; `False` to ignore

        Returns
        - applied options
        """
        ...


    def isEnumCaseSensitive(self) -> bool:
        """
        getter

        Returns
        - True when parsing enum case-sensitive
        """
        ...


    def setEnumCaseSensitive(self, enumCaseSensitive: bool) -> None:
        """
        Disables or enables case sensitivity during construct enum constant from string value Default
        is False.

        Arguments
        - enumCaseSensitive: - True to set enum case-sensitive, False the reverse
        """
        ...


    def getNestingDepthLimit(self) -> int:
        """
        getter

        Returns
        - the limit
        """
        ...


    def setNestingDepthLimit(self, nestingDepthLimit: int) -> None:
        """
        Set max depth of nested collections. When the limit is exceeded an exception is thrown.
        Aliases/Anchors are not counted. This is to prevent a DoS attack

        Arguments
        - nestingDepthLimit: - depth to be accepted (50 by default)
        """
        ...


    def getCodePointLimit(self) -> int:
        """
        getter

        Returns
        - max code points in the input document
        """
        ...


    def setCodePointLimit(self, codePointLimit: int) -> None:
        """
        The max amount of code points for every input YAML document in the stream. Please be aware that
        byte limit depends on the encoding.

        Arguments
        - codePointLimit: - the max allowed size of a single YAML document in a stream
        """
        ...


    def getTagInspector(self) -> "TagInspector":
        ...


    def setTagInspector(self, tagInspector: "TagInspector") -> None:
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.extensions.compactnotation.CompactConstructor

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Iterator
from java.util.regex import Matcher
from java.util.regex import Pattern
from org.yaml.snakeyaml import LoaderOptions
from org.yaml.snakeyaml.constructor import Construct
from org.yaml.snakeyaml.constructor import Constructor
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.extensions.compactnotation import *
from org.yaml.snakeyaml.introspector import Property
from org.yaml.snakeyaml.nodes import MappingNode
from org.yaml.snakeyaml.nodes import Node
from org.yaml.snakeyaml.nodes import NodeTuple
from org.yaml.snakeyaml.nodes import ScalarNode
from org.yaml.snakeyaml.nodes import SequenceNode
from typing import Any, Callable, Iterable, Tuple


class CompactConstructor(Constructor):
    """
    Construct a custom Java instance out of a compact object notation format.
    """

    def __init__(self, loadingConfig: "LoaderOptions"):
        """
        Create with provided options

        Arguments
        - loadingConfig: - options
        """
        ...


    def __init__(self):
        """
        Create with defaults
        """
        ...


    def getCompactData(self, scalar: str) -> "CompactData":
        ...


    class ConstructCompactObject(ConstructMapping):
        """
        Custom ConstructMapping
        """

        def construct2ndStep(self, node: "Node", object: "Object") -> None:
            ...


        def construct(self, node: "Node") -> "Object":
            ...
